#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import io
import json
import tempfile

import fixtures
import mock
import six

from tests import common
from tests import heat_config_notify as hcn


class HeatConfigNotifyTest(common.RunScriptTest):

    data_signal_id = {
        'id': '5555',
        'group': 'script',
        'inputs': [{
            'name': 'deploy_signal_id',
            'value': 'mock://192.0.2.3/foo'
        }],
        'config': 'five'
    }

    data_signal_id_put = {
        'id': '5555',
        'group': 'script',
        'inputs': [{
            'name': 'deploy_signal_id',
            'value': 'mock://192.0.2.3/foo'
        }, {
            'name': 'deploy_signal_verb',
            'value': 'PUT'
        }],
        'config': 'five'
    }

    data_heat_signal = {
        'id': '5555',
        'group': 'script',
        'inputs': [{
            'name': 'deploy_auth_url',
            'value': 'mock://192.0.2.3/auth'
        }, {
            'name': 'deploy_user_id',
            'value': 'aaaa'
        }, {
            'name': 'deploy_password',
            'value': 'password'
        }, {
            'name': 'deploy_project_id',
            'value': 'bbbb'
        }, {
            'name': 'deploy_stack_id',
            'value': 'cccc'
        }, {
            'name': 'deploy_resource_name',
            'value': 'the_resource'
        }, {
            'name': 'deploy_region_name',
            'value': 'RegionOne'
        }],
        'config': 'five'
    }

    def setUp(self):
        super(HeatConfigNotifyTest, self).setUp()
        self.deployed_dir = self.useFixture(fixtures.TempDir())
        hcn.init_logging = mock.MagicMock()
        if six.PY2:
            self.stdin = io.BytesIO()
        else:
            self.stdin = io.StringIO()

    def write_config_file(self, data):
        config_file = tempfile.NamedTemporaryFile(mode='w')
        config_file.write(json.dumps(data))
        config_file.flush()
        return config_file

    def test_notify_missing_file(self):

        signal_data = json.dumps({'foo': 'bar'})
        self.stdin.write(signal_data)
        self.stdin.seek(0)

        with self.write_config_file(self.data_signal_id) as config_file:
            config_file_name = config_file.name

        self.assertEqual(
            1, hcn.main(['heat-config-notify', config_file_name], self.stdin))

    def test_notify_missing_file_arg(self):

        signal_data = json.dumps({'foo': 'bar'})
        self.stdin.write(signal_data)
        self.stdin.seek(0)

        self.assertEqual(
            1, hcn.main(['heat-config-notify'], self.stdin))

    def test_notify_signal_id(self):
        requests = mock.MagicMock()
        session = mock.MagicMock()
        requests.Session.return_value = session
        retry = mock.MagicMock()
        httpadapter = mock.MagicMock()

        hcn.requests = requests
        hcn.Retry = retry
        hcn.HTTPAdapter = httpadapter

        signal_data = json.dumps({'foo': 'bar'})
        self.stdin.write(signal_data)
        self.stdin.seek(0)

        with self.write_config_file(self.data_signal_id) as config_file:
            self.assertEqual(
                0,
                hcn.main(['heat-config-notify', config_file.name], self.stdin))

        session.post.assert_called_once_with(
            'mock://192.0.2.3/foo',
            data=signal_data,
            headers={'content-type': 'application/json'})

    def test_notify_signal_id_put(self):
        requests = mock.MagicMock()
        session = mock.MagicMock()
        requests.Session.return_value = session
        retry = mock.MagicMock()
        httpadapter = mock.MagicMock()

        hcn.requests = requests
        hcn.Retry = retry
        hcn.HTTPAdapter = httpadapter

        session.post.return_value = '[200]'

        signal_data = json.dumps({'foo': 'bar'})
        self.stdin.write(signal_data)
        self.stdin.seek(0)

        with self.write_config_file(self.data_signal_id_put) as config_file:
            self.assertEqual(
                0,
                hcn.main(['heat-config-notify', config_file.name], self.stdin))

        session.put.assert_called_once_with(
            'mock://192.0.2.3/foo',
            data=signal_data,
            headers={'content-type': 'application/json'})

    def test_notify_signal_id_empty_data(self):
        requests = mock.MagicMock()
        session = mock.MagicMock()
        requests.Session.return_value = session
        retry = mock.MagicMock()
        httpadapter = mock.MagicMock()

        hcn.requests = requests
        hcn.Retry = retry
        hcn.HTTPAdapter = httpadapter

        session.post.return_value = '[200]'

        with self.write_config_file(self.data_signal_id) as config_file:
            self.assertEqual(
                0,
                hcn.main(['heat-config-notify', config_file.name], self.stdin))

        session.post.assert_called_once_with(
            'mock://192.0.2.3/foo',
            data='{}',
            headers={'content-type': 'application/json'})

    def test_notify_signal_id_invalid_json_data(self):
        requests = mock.MagicMock()
        session = mock.MagicMock()
        requests.Session.return_value = session
        retry = mock.MagicMock()
        httpadapter = mock.MagicMock()

        hcn.requests = requests
        hcn.Retry = retry
        hcn.HTTPAdapter = httpadapter

        session.post.return_value = '[200]'

        signal_data = json.dumps({'foo': 'bar'})
        self.stdin.write(signal_data)
        self.stdin.write(signal_data[:-3])
        self.stdin.seek(0)

        with self.write_config_file(self.data_signal_id) as config_file:
            self.assertEqual(
                0,
                hcn.main(['heat-config-notify', config_file.name], self.stdin))

        session.post.assert_called_once_with(
            'mock://192.0.2.3/foo',
            data='{}',
            headers={'content-type': 'application/json'})

    def test_notify_heat_signal(self):
        ksclient = mock.MagicMock()
        hcn.ksclient = ksclient
        ks = mock.MagicMock()
        ksclient.Client.return_value = ks

        heatclient = mock.MagicMock()
        hcn.heatclient = heatclient
        heat = mock.MagicMock()
        heatclient.Client.return_value = heat

        signal_data = json.dumps({'foo': 'bar'})
        self.stdin.write(signal_data)
        self.stdin.seek(0)

        ks.service_catalog.url_for.return_value = 'mock://192.0.2.3/heat'
        heat.resources.signal.return_value = 'all good'

        with self.write_config_file(self.data_heat_signal) as config_file:
            self.assertEqual(
                0,
                hcn.main(['heat-config-notify', config_file.name], self.stdin))

        ksclient.Client.assert_called_once_with(
            auth_url='mock://192.0.2.3/auth',
            user_id='aaaa',
            password='password',
            project_id='bbbb')
        ks.service_catalog.url_for.assert_called_once_with(
            service_type='orchestration', endpoint_type='publicURL',
            region_name='RegionOne')

        heatclient.Client.assert_called_once_with(
            '1', 'mock://192.0.2.3/heat', token=ks.auth_token)
        heat.resources.signal.assert_called_once_with(
            'cccc',
            'the_resource',
            data={'foo': 'bar'})



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy
import json
import os
import shutil
import tempfile

import fixtures
from testtools import matchers

from tests import common


class HeatConfigTest(common.RunScriptTest):

    fake_hooks = ['cfn-init', 'chef', 'puppet', 'salt', 'script',
                  'apply-config', 'hiera', 'json-file']

    data = [
        {
            'id': '1111',
            'group': 'chef',
            'inputs': [{
                'name': 'deploy_signal_id',
                'value': 'mock://192.0.2.2/foo'
            }],
            'config': 'one'
        }, {
            'id': '2222',
            'group': 'cfn-init',
            'inputs': [],
            'config': 'two'
        }, {
            'id': '3333',
            'group': 'salt',
            'inputs': [{'name': 'foo', 'value': 'bar'}],
            'outputs': [{'name': 'foo'}],
            'config': 'three'
        }, {
            'id': '4444',
            'group': 'puppet',
            'inputs': [],
            'config': 'four'
        }, {
            'id': '5555',
            'group': 'script',
            'inputs': [{
                'name': 'deploy_status_code', 'value': '-1'
            }, {
                'name': 'deploy_stderr', 'value': 'A bad thing happened'
            }, {
                'name': 'deploy_signal_id',
                'value': 'mock://192.0.2.3/foo'
            }],
            'config': 'five'
        }, {
            'id': '6666',
            'group': 'apply-config',
            'inputs': [{'name': 'foo', 'value': 'bar'}],
            'config': 'six'
        }, {
            'id': '7777',
            'group': 'hiera',
            'inputs': [],
            'config': 'seven'
        }, {
            'id': '8888',
            'group': 'json-file',
            'inputs': [],
            'config': 'eight'
        }, {
            'id': '9999',
            'group': 'no-such-hook',
            'inputs': [],
            'config': 'nine'
        }]

    outputs = {
        'chef': {
            'deploy_status_code': '0',
            'deploy_stderr': 'stderr',
            'deploy_stdout': 'stdout'
        },
        'cfn-init': {
            'deploy_status_code': '0',
            'deploy_stderr': 'stderr',
            'deploy_stdout': 'stdout'
        },
        'salt': {
            'deploy_status_code': '0',
            'deploy_stderr': 'stderr',
            'deploy_stdout': 'stdout',
            'foo': 'bar'
        },
        'puppet': {
            'deploy_status_code': '0',
            'deploy_stderr': 'stderr',
            'deploy_stdout': 'stdout'
        },
        'script': {
            'deploy_status_code': '-1',
            'deploy_stderr': 'A bad thing happened',
            'deploy_stdout': 'stdout'
        },
        'hiera': {
            'deploy_status_code': '0',
            'deploy_stderr': 'stderr',
            'deploy_stdout': 'stdout'
        },
        'json-file': {
            'deploy_status_code': '0',
            'deploy_stderr': 'stderr',
            'deploy_stdout': 'stdout'
        },
        'apply-config': {
            'deploy_status_code': '0',
            'deploy_stderr': 'stderr',
            'deploy_stdout': 'stdout'
        }
    }

    def setUp(self):
        super(HeatConfigTest, self).setUp()

        self.fake_hook_path = self.relative_path(__file__, 'hook-fake.py')

        self.heat_config_path = self.relative_path(
            __file__,
            '..',
            'heat-config/os-refresh-config/configure.d/55-heat-config')

        self.hooks_dir = self.useFixture(fixtures.TempDir())
        self.deployed_dir = self.useFixture(fixtures.TempDir())

        with open(self.fake_hook_path) as f:
            fake_hook = f.read()

        for hook in self.fake_hooks:
            hook_name = self.hooks_dir.join(hook)
            with open(hook_name, 'w') as f:
                os.utime(hook_name, None)
                f.write(fake_hook)
                f.flush()
            os.chmod(hook_name, 0o755)
        self.env = os.environ.copy()

    def write_config_file(self, data):
        config_file = tempfile.NamedTemporaryFile(mode='w')
        config_file.write(json.dumps(data))
        config_file.flush()
        return config_file

    def run_heat_config(self, data):
        with self.write_config_file(data) as config_file:

            self.env.update({
                'HEAT_CONFIG_HOOKS': self.hooks_dir.join(),
                'HEAT_CONFIG_DEPLOYED': self.deployed_dir.join(),
                'HEAT_SHELL_CONFIG': config_file.name
            })
            returncode, stdout, stderr = self.run_cmd(
                [self.heat_config_path], self.env)

            self.assertEqual(0, returncode, stderr)

    def test_hooks_exist(self):
        self.assertThat(
            self.hooks_dir.join('no-such-hook'),
            matchers.Not(matchers.FileExists()))

        for hook in self.fake_hooks:
            hook_path = self.hooks_dir.join(hook)
            self.assertThat(hook_path, matchers.FileExists())

    def test_run_heat_config(self):

        self.run_heat_config(self.data)

        for config in self.data:
            hook = config['group']
            stdin_path = self.hooks_dir.join('%s.stdin' % hook)
            stdout_path = self.hooks_dir.join('%s.stdout' % hook)
            deployed_file = self.deployed_dir.join('%s.json' % config['id'])

            if hook == 'no-such-hook':
                self.assertThat(
                    stdin_path, matchers.Not(matchers.FileExists()))
                self.assertThat(
                    stdout_path, matchers.Not(matchers.FileExists()))
                continue

            self.assertThat(stdin_path, matchers.FileExists())
            self.assertThat(stdout_path, matchers.FileExists())

            # parsed stdin should match the config item
            self.assertEqual(config,
                             self.json_from_file(stdin_path))

            # parsed stdin should match the written deployed file
            self.assertEqual(config,
                             self.json_from_file(deployed_file))

            self.assertEqual(self.outputs[hook],
                             self.json_from_file(stdout_path))

            # clean up files in preparation for second run
            os.remove(stdin_path)
            os.remove(stdout_path)

        # run again with no changes, assert no new files
        self.run_heat_config(self.data)
        for config in self.data:
            hook = config['group']
            stdin_path = self.hooks_dir.join('%s.stdin' % hook)
            stdout_path = self.hooks_dir.join('%s.stdout' % hook)

            self.assertThat(
                stdin_path, matchers.Not(matchers.FileExists()))
            self.assertThat(
                stdout_path, matchers.Not(matchers.FileExists()))

        # run again changing the puppet config
        data = copy.deepcopy(self.data)
        for config in data:
            if config['id'] == '4444':
                config['id'] = '44444444'
        self.run_heat_config(data)
        for config in self.data:
            hook = config['group']
            stdin_path = self.hooks_dir.join('%s.stdin' % hook)
            stdout_path = self.hooks_dir.join('%s.stdout' % hook)

            if hook == 'puppet':
                self.assertThat(stdin_path, matchers.FileExists())
                self.assertThat(stdout_path, matchers.FileExists())
            else:
                self.assertThat(
                    stdin_path, matchers.Not(matchers.FileExists()))
                self.assertThat(
                    stdout_path, matchers.Not(matchers.FileExists()))

        # run again with a different deployed_dir
        old_deployed_dir = self.deployed_dir
        self.env['HEAT_CONFIG_DEPLOYED_OLD'] = old_deployed_dir.join()
        self.deployed_dir = self.useFixture(fixtures.TempDir())
        # make sure the new deployed_dir doesn't exist to trigger the migration
        shutil.rmtree(self.deployed_dir.join())

        self.run_heat_config(data)
        for config in self.data:
            hook = config['group']
            if hook == 'no-such-hook':
                continue
            deployed_file = self.deployed_dir.join('%s.json' % config['id'])
            old_deployed_file = old_deployed_dir.join('%s.json' % config['id'])
            self.assertEqual(config,
                             self.json_from_file(deployed_file))
            self.assertThat(
                old_deployed_file, matchers.Not(matchers.FileExists()))



#!/usr/bin/env python
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import logging
import os
import subprocess
import sys

APPLY_CONFIG_CMD = os.environ.get('HEAT_APPLY_CONFIG_CMD', 'os-apply-config')


def main(argv=sys.argv):
    log = logging.getLogger('heat-config')
    handler = logging.StreamHandler(sys.stderr)
    handler.setFormatter(
        logging.Formatter(
            '[%(asctime)s] (%(name)s) [%(levelname)s] %(message)s'))
    log.addHandler(handler)
    log.setLevel('DEBUG')

    env = os.environ.copy()

    log.debug('Running %s' % APPLY_CONFIG_CMD)
    subproc = subprocess.Popen([APPLY_CONFIG_CMD], stdout=subprocess.PIPE,
                               stderr=subprocess.PIPE, env=env)
    stdout, stderr = subproc.communicate()

    log.info(stdout)
    log.debug(stderr)

    if subproc.returncode:
        log.error("Error running apply-config: [%s]\n" % subproc.returncode)
    else:
        log.info('Completed apply-config.')

    response = {
        'deploy_stdout': stdout.decode('utf-8', 'replace'),
        'deploy_stderr': stderr.decode('utf-8', 'replace'),
        'deploy_status_code': subproc.returncode,
    }

    json.dump(response, sys.stdout)

if __name__ == '__main__':
    sys.exit(main(sys.argv))



#!/usr/bin/env python
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import collections
import json
import logging
import os
import sys

import paunch
import yaml

DOCKER_CMD = os.environ.get('HEAT_DOCKER_CMD', 'docker')


log = None


def build_response(deploy_stdout, deploy_stderr, deploy_status_code):
    return {
        'deploy_stdout': deploy_stdout,
        'deploy_stderr': deploy_stderr,
        'deploy_status_code': deploy_status_code,
    }


def main(argv=sys.argv):
    global log
    log = logging.getLogger('heat-config')
    handler = logging.StreamHandler(sys.stderr)
    handler.setFormatter(
        logging.Formatter(
            '[%(asctime)s] (%(name)s) [%(levelname)s] %(message)s'))
    log.addHandler(handler)
    log.setLevel('DEBUG')

    c = json.load(sys.stdin)

    input_values = dict((i['name'], i['value']) for i in c.get('inputs', {}))

    if input_values.get('deploy_action') == 'DELETE':
        json.dump(build_response(
            '', '', 0), sys.stdout)
        return

    config = c.get('config', '')
    cid = c.get('id')
    if not config:
        log.debug("No 'config' input found, nothing to do.")
        json.dump(build_response(
            '', '', 0), sys.stdout)
        return

    stdout = []
    stderr = []
    deploy_status_code = 0

    # convert config to dict
    if not isinstance(config, dict):
        config = yaml.safe_load(config)

    labels = collections.OrderedDict()
    labels['deploy_stack_id'] = input_values.get('deploy_stack_id')
    labels['deploy_resource_name'] = input_values.get('deploy_resource_name')
    stdout, stderr, deploy_status_code = paunch.apply(
        cid,
        config,
        managed_by='docker-cmd',
        labels=labels,
        docker_cmd=DOCKER_CMD
    )

    json.dump(build_response(
        '\n'.join(stdout), '\n'.join(stderr), deploy_status_code), sys.stdout)

if __name__ == '__main__':
    sys.exit(main(sys.argv))



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import os
import tempfile

import fixtures
import yaml

from tests import common


class HeatConfigDockerComposeORCTest(common.RunScriptTest):

    fake_hooks = ['docker-compose']

    data = [
        {
            "name": "abcdef001",
            "group": "docker-compose",
            "inputs": {},
            "config": {
                "web": {
                    "image": "nginx",
                    "links": [
                        "db"
                    ],
                    "ports": [
                        "8000:8000"
                    ]
                },
                "db": {
                    "image": "redis"
                }
            }
        },
        {
            "name": "abcdef002",
            "group": "docker-compose",
            "inputs": {},
            "config": {
                "web": {
                    "image": "httpd",
                    "links": [
                        "db"
                    ],
                    "ports": [
                        "80:8001"
                    ]
                },
                "db": {
                    "image": "postgress"
                }
            }
        }
    ]

    def setUp(self):
        super(HeatConfigDockerComposeORCTest, self).setUp()

        self.fake_hook_path = self.relative_path(__file__, 'hook-fake.py')
        self.heat_config_docker_compose_path = self.relative_path(
            __file__,
            '..',
            'heat-config-docker-compose/os-refresh-config/configure.d/'
            '50-heat-config-docker-compose')

        self.docker_compose_dir = self.useFixture(fixtures.TempDir())

        with open(self.fake_hook_path) as f:
            fake_hook = f.read()

        for hook in self.fake_hooks:
            hook_name = self.docker_compose_dir.join(hook)
            with open(hook_name, 'w') as f:
                os.utime(hook_name, None)
                f.write(fake_hook)
                f.flush()
            os.chmod(hook_name, 0o755)

    def write_config_file(self, data):
        config_file = tempfile.NamedTemporaryFile(mode='w')
        config_file.write(json.dumps(data))
        config_file.flush()
        return config_file

    def test_run_heat_config(self):
        with self.write_config_file(self.data) as config_file:
            env = os.environ.copy()
            env.update({
                'HEAT_DOCKER_COMPOSE_WORKING': self.docker_compose_dir.join(),
                'HEAT_SHELL_CONFIG': config_file.name
            })

            returncode, stdout, stderr = self.run_cmd(
                [self.heat_config_docker_compose_path], env)

            self.assertEqual(0, returncode, stderr)

            compose_yml = self.docker_compose_dir.join(
                'abcdef001/docker-compose.yml')
            with open(compose_yml) as f:
                    self.assertEqual(yaml.safe_dump(
                        self.data[0].get('config'),
                        default_flow_style=False), f.read())



#!/usr/bin/env python
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import logging
import os
import re
import subprocess
import sys


WORKING_DIR = os.environ.get('HEAT_PUPPET_WORKING',
                             '/var/lib/heat-config/heat-config-puppet')
OUTPUTS_DIR = os.environ.get('HEAT_PUPPET_OUTPUTS',
                             '/var/run/heat-config/heat-config-puppet')
PUPPET_CMD = os.environ.get('HEAT_PUPPET_CMD', 'puppet')
PUPPET_LOGDIR = os.environ.get(
    'HEAT_PUPPET_LOGDIR', '/var/run/heat-config/deployed'
)
HIERA_DATADIR = os.environ.get('HEAT_PUPPET_HIERA_DATADIR',
                               '/etc/puppet/hieradata')


def prepare_dir(path):
    if not os.path.isdir(path):
        os.makedirs(path, 0o700)


def get_hostname_f(log):
    subproc = subprocess.Popen(['hostname', '-f'], stdout=subprocess.PIPE)
    out = subproc.communicate()[0]
    if subproc.returncode == 0:
        return out.strip()
    else:
        log.warn("Failed to retrieve 'hostname -f' output")
        return None


def main(argv=sys.argv):
    log = logging.getLogger('heat-config')
    handler = logging.StreamHandler(sys.stderr)
    handler.setFormatter(
        logging.Formatter(
            '[%(asctime)s] (%(name)s) [%(levelname)s] %(message)s'))
    log.addHandler(handler)
    log.setLevel('DEBUG')

    prepare_dir(OUTPUTS_DIR)
    prepare_dir(WORKING_DIR)
    os.chdir(WORKING_DIR)

    c = json.load(sys.stdin)

    use_hiera = c['options'].get('enable_hiera', False)
    use_facter = c['options'].get('enable_facter', True)
    modulepath = c['options'].get('modulepath')
    tags = c['options'].get('tags')
    debug = c['options'].get('enable_debug', False)
    verbose = c['options'].get('enable_verbose', False)

    facts = {}
    hiera = {}

    fqdn = get_hostname_f(log)
    if fqdn:
        facts['FACTER_fqdn'] = fqdn

    for input in c['inputs']:
        input_name = input['name']
        input_value = input.get('value', '')
        if use_facter:
            fact_name = 'FACTER_%s' % input_name
            facts[fact_name] = input_value
        if use_hiera:
            hiera[input_name] = input_value

    if use_hiera:
        prepare_dir(HIERA_DATADIR)
        hiera_data = os.path.join(HIERA_DATADIR,
                                  'heat_config_%s.json' % c['name'])
        with os.fdopen(os.open(hiera_data,
                               os.O_CREAT | os.O_TRUNC | os.O_WRONLY, 0o600),
                       'w') as hiera_file:
            hiera_file.write(json.dumps(hiera))
        facts['FACTER_deploy_config_name'] = c['name']

    fn = os.path.join(WORKING_DIR, '%s.pp' % c['id'])
    heat_outputs_path = os.path.join(OUTPUTS_DIR, c['id'])
    facts['FACTER_heat_outputs_path'] = heat_outputs_path

    env_debug = ' '.join('%s="%s" ' % (k, v) for k, v in facts.items())

    env = os.environ.copy()
    env.update(facts)

    with os.fdopen(os.open(fn, os.O_CREAT | os.O_TRUNC | os.O_WRONLY, 0o700),
                   'w') as f:
        f.write(c.get('config', ''))

    cmd = [PUPPET_CMD, 'apply', '--color', 'false', '--detailed-exitcodes', fn]
    # This is the default log destination to print out to the console and
    # captured by heat via the subprocess method below.
    cmd.insert(-1, '--logdest')
    cmd.insert(-1, 'console')
    if modulepath:
        cmd.insert(-1, '--modulepath')
        cmd.insert(-1, modulepath)
    if tags:
        cmd.insert(-1, '--tags')
        cmd.insert(-1, tags)
    if debug:
        cmd.insert(-1, '--debug')
        cmd.insert(-1, '--logdest')
        cmd.insert(-1, '/var/log/puppet/heat-debug.log')
    if verbose:
        cmd.insert(-1, '--verbose')
        cmd.insert(-1, '--logdest')
        cmd.insert(-1, '/var/log/puppet/heat-verbose.log')

    prepare_dir(PUPPET_LOGDIR)
    timestamp = re.sub('[:T]', '-', c['creation_time'])
    base_path = os.path.join(
        PUPPET_LOGDIR, '{timestamp}-{c[id]}'.format(**locals())
    )
    stdout_log = open('{0}-stdout.log'.format(base_path), 'w')
    stderr_log = open('{0}-stderr.log'.format(base_path), 'w')
    log.debug('Running %s %s' % (env_debug, ' '.join(cmd)))
    try:
        subproc = subprocess.Popen(
            cmd, stdout=stdout_log, stderr=stderr_log, env=env
        )
        subproc.wait()
    except OSError:
        log.warn('puppet not installed yet')
        return
    finally:
        stdout_log.close()
        stderr_log.close()

    log.info('Return code %s' % subproc.returncode)
    response = {}
    for i in 'stdout', 'stderr':
        with open('{0}-{1}.log'.format(base_path, i)) as logfile:
            content = logfile.read()
        if content.strip():
            log.info(content)
        response['deploy_{0}'.format(i)] = content

    # returncode of 2 means there were successful changes
    if subproc.returncode in (0, 2):
        returncode = 0
        log.info('Completed %s' % fn)
    else:
        returncode = subproc.returncode
        log.error("Error running %s. [%s]\n" % (fn, subproc.returncode))

    for output in c.get('outputs') or []:
        output_name = output['name']
        try:
            with open('%s.%s' % (heat_outputs_path, output_name)) as out:
                response[output_name] = out.read()
        except IOError:
            pass

    response.update({
        'deploy_status_code': returncode,
    })
    json.dump(response, sys.stdout)


if __name__ == '__main__':
    sys.exit(main(sys.argv))



#!/usr/bin/env python
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import logging
import os
import sys

import salt.cli.caller
import salt.config
from salt import exceptions
import yaml


WORKING_DIR = os.environ.get('HEAT_SALT_WORKING',
                             '/var/lib/heat-config/heat-config-salt')
SALT_MINION_CONFIG = os.environ.get('SALT_MINION_CONFIG',
                                    '/etc/salt/minion')


def prepare_dir(path):
    if not os.path.isdir(path):
        os.makedirs(path, 0o700)


def main(argv=sys.argv):
    log = logging.getLogger('heat-config')
    handler = logging.StreamHandler(sys.stderr)
    handler.setFormatter(
        logging.Formatter(
            '[%(asctime)s] (%(name)s) [%(levelname)s] %(message)s'))
    log.addHandler(handler)
    log.setLevel('DEBUG')

    prepare_dir(WORKING_DIR)
    os.chdir(WORKING_DIR)

    c = json.load(sys.stdin)

    opts = salt.config.minion_config(SALT_MINION_CONFIG)

    opts['file_roots'] = {'base': [WORKING_DIR]}
    opts['file_client'] = 'local'
    opts['local'] = 'local'
    opts['fun'] = 'state.sls'
    opts['arg'] = [c['id']]

    for input in c['inputs']:
        key = input['name']
        opts[key] = input.get('value', '')

    state_file = '%s.sls' % c['id']
    config = c.get('config', '')

    if isinstance(config, dict):
        yaml_config = yaml.safe_dump(config, default_flow_style=False)
    else:
        yaml_config = config

    fn = os.path.join(WORKING_DIR, state_file)
    with os.fdopen(os.open(fn, os.O_CREAT | os.O_WRONLY, 0o700), 'w') as f:
        f.write(yaml_config)

    caller = salt.cli.caller.Caller.factory(opts)

    log.debug('Applying Salt state %s' % state_file)

    stdout, stderr = None, None
    ret = {}

    try:
        ret = caller.call()
    except exceptions.SaltInvocationError as err:
        log.error(
            'Salt invocation error while applying Salt sate %s' % state_file)
        stderr = err

    if ret:

        log.info('Results: %s' % ret)
        output = yaml.safe_dump(ret['return'])

        # returncode of 0 means there were successful changes
        if ret['retcode'] == 0:
            log.info('Completed applying salt state %s' % state_file)
            stdout = output
        else:
            # Salt doesn't always return sane return codes so we have to check
            # individual results
            runfailed = False
            for state, data in ret['return'].items():
                if not data['result']:
                    runfailed = True
                    break
            if runfailed:
                log.error('Error applying Salt state %s. [%s]\n'
                          % (state_file, ret['retcode']))
                stderr = output
            else:
                ret['retcode'] = 0
                stdout = output

    response = {}

    for output in c.get('outputs', []):
        output_name = output['name']
        response[output_name] = ret.get(output_name)

    response.update({
        'deploy_stdout': stdout,
        'deploy_stderr': stderr,
        'deploy_status_code': ret['retcode'],
    })
    json.dump(response, sys.stdout)

if __name__ == '__main__':
    sys.exit(main(sys.argv))



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import os

import fixtures

from tests import common


class HookScriptTest(common.RunScriptTest):

    def setUp(self):
        super(HookScriptTest, self).setUp()
        self.hook_path = self.relative_path(
            __file__,
            '..',
            'heat-config-script/install.d/hook-script.py')

        self.fake_tool_path = self.relative_path(
            __file__,
            'config-tool-fake.py')

        with open(self.fake_tool_path) as f:
            self.fake_tool_contents = f.read()

        self.data = {
            'id': '1234',
            'group': 'script',
            'inputs': [
                {'name': 'foo', 'value': 'bar'},
                {'name': 'another', 'value': 'input'},
                {'name': 'a_dict', 'value': '{"key": "value"}'},
                {'name': 'a_list', 'value': '["v1", 12]'},
            ],
            'outputs': [
                {'name': 'first_output'},
                {'name': 'second_output'}
            ],
            'config': self.fake_tool_contents
        }

        self.working_dir = self.useFixture(fixtures.TempDir())
        self.outputs_dir = self.useFixture(fixtures.TempDir())
        self.test_state_path = self.outputs_dir.join('test_state.json')

        self.env = os.environ.copy()
        self.env.update({
            'HEAT_SCRIPT_WORKING': self.working_dir.join(),
            'HEAT_SCRIPT_OUTPUTS': self.outputs_dir.join(),
            'TEST_STATE_PATH': self.test_state_path,
        })

    def test_hook(self):

        self.env.update({
            'TEST_RESPONSE': json.dumps({
                'stdout': 'script success',
                'stderr': 'thing happened',
                'files': {
                    self.outputs_dir.join('1234.first_output'): 'output 1',
                    self.outputs_dir.join('1234.second_output'): 'output 2',
                }
            }),
        })
        returncode, stdout, stderr = self.run_cmd(
            [self.hook_path], self.env, json.dumps(self.data))

        self.assertEqual(0, returncode, stderr)
        self.assertEqual({
            'deploy_stdout': 'script success',
            'deploy_stderr': 'thing happened',
            'deploy_status_code': 0,
            'first_output': 'output 1',
            'second_output': 'output 2',
        }, json.loads(stdout.decode('utf-8')))

        state = self.json_from_file(self.test_state_path)
        script = self.working_dir.join('1234')
        with open(script) as f:
            self.assertEqual(self.fake_tool_contents, f.read())

        self.assertEqual([script], state['args'])
        self.assertEqual('bar', state['env']['foo'])
        self.assertEqual('input', state['env']['another'])
        self.assertEqual('{"key": "value"}', state['env']['a_dict'])
        self.assertEqual('["v1", 12]', state['env']['a_list'])
        self.assertEqual(self.outputs_dir.join('1234'),
                         state['env']['heat_outputs_path'])

    def test_hook_failed(self):

        self.env.update({
            'TEST_RESPONSE': json.dumps({
                'stdout': 'script failed',
                'stderr': 'bad thing happened',
                'returncode': 1
            }),
        })
        returncode, stdout, stderr = self.run_cmd(
            [self.hook_path], self.env, json.dumps(self.data))

        self.assertEqual(0, returncode, stderr)
        self.assertEqual({
            'deploy_stdout': 'script failed',
            'deploy_stderr': 'bad thing happened',
            'deploy_status_code': 1,
        }, json.loads(stdout.decode('utf-8')))

        state = self.json_from_file(self.test_state_path)
        script = self.working_dir.join('1234')
        with open(script) as f:
            self.assertEqual(self.fake_tool_contents, f.read())

        self.assertEqual([script], state['args'])
        self.assertEqual('bar', state['env']['foo'])
        self.assertEqual('input', state['env']['another'])
        self.assertEqual('{"key": "value"}', state['env']['a_dict'])
        self.assertEqual('["v1", 12]', state['env']['a_list'])
        self.assertEqual(self.outputs_dir.join('1234'),
                         state['env']['heat_outputs_path'])



#!/usr/bin/env python
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import logging
import os
import subprocess
import sys


# Ideally this path would be /var/lib/heat-cfntools/cfn-init-data
# but this is where all boot metadata is stored
LAST_METADATA_DIR = os.environ.get('HEAT_CFN_INIT_LAST_METADATA_DIR',
                                   '/var/cache/heat-cfntools')


CFN_INIT_CMD = os.environ.get('HEAT_CFN_INIT_CMD',
                              'cfn-init')


def main(argv=sys.argv, stdin=sys.stdin, stdout=sys.stdout, stderr=sys.stderr):
    log = logging.getLogger('heat-config')
    handler = logging.StreamHandler(stderr)
    handler.setFormatter(
        logging.Formatter(
            '[%(asctime)s] (%(name)s) [%(levelname)s] %(message)s'))
    log.addHandler(handler)
    log.setLevel('DEBUG')

    c = json.load(stdin)

    config = c.get('config', {})
    if not isinstance(config, dict):
        config = json.loads(config)
    meta = {'AWS::CloudFormation::Init': config}

    if not os.path.isdir(LAST_METADATA_DIR):
        os.makedirs(LAST_METADATA_DIR, 0o700)

    fn = os.path.join(LAST_METADATA_DIR, 'last_metadata')
    with os.fdopen(os.open(fn, os.O_CREAT | os.O_WRONLY | os.O_TRUNC, 0o700),
                   'w') as f:
        json.dump(meta, f)

    log.debug('Running %s' % CFN_INIT_CMD)
    subproc = subprocess.Popen([CFN_INIT_CMD], stdout=subprocess.PIPE,
                               stderr=subprocess.PIPE)
    cstdout, cstderr = subproc.communicate()

    if cstdout:
        log.info(cstdout)
    if cstderr:
        log.info(cstderr)

    if subproc.returncode:
        log.error("Error running %s. [%s]\n" % (
            CFN_INIT_CMD, subproc.returncode))
    else:
        log.info('Completed %s' % CFN_INIT_CMD)

    response = {
        'deploy_stdout': cstdout.decode('utf-8', 'replace'),
        'deploy_stderr': cstderr.decode('utf-8', 'replace'),
        'deploy_status_code': subproc.returncode,
    }

    json.dump(response, stdout)


if __name__ == '__main__':
    sys.exit(main())



#!/usr/bin/env python
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import io
import json
import logging
import os
import re
import six
import sys
import time

try:
    import docker
except ImportError:
    docker = None


DOCKER_BASE_URL = os.environ.get('DOCKER_HOST',
                                 'unix:///var/run/docker.sock')


DEFAULT_IMAGES_TIMEOUT = 600


DEFAULT_CONTAINERS_TIMEOUT = 120


DEFAULT_POLL_PERIOD = 5


def get_client(log):
    kwargs = {}
    kwargs['base_url'] = DOCKER_BASE_URL
    log.debug('Connecting to %s' % DOCKER_BASE_URL)
    client = docker.Client(**kwargs)
    client._version = client.version()['ApiVersion']
    log.debug('Connected to version %s' % client._version)
    return client


def id_to_pod_name_part(config_id):
    return config_id.replace('-', '')[:15]


def container_pattern(config_id, container_name):
    return '^/k8s_%s\.[0-9a-z]{8}_%s' % (
        container_name, id_to_pod_name_part(config_id))


def required_images(c):
    containers = c['config'].get('containers', [])
    return set(container['image'] for container in containers)


def required_container_patterns(c):
    config_id = c['id']
    containers = c['config'].get('containers', [])
    return dict((container['name'], container_pattern(
        config_id, container['name'])) for container in containers)


def configure_logging():
    log = logging.getLogger('heat-config')
    log.setLevel('DEBUG')
    formatter = logging.Formatter(
        '[%(asctime)s] (%(name)s) [%(levelname)s] %(message)s')

    # debug log to stderr
    handler = logging.StreamHandler(sys.stderr)
    handler.setFormatter(formatter)
    log.addHandler(handler)

    deploy_stdout = io.StringIO()
    handler = logging.StreamHandler(deploy_stdout)
    handler.setFormatter(formatter)
    handler.setLevel('DEBUG')
    log.addHandler(handler)

    deploy_stderr = io.StringIO()
    handler = logging.StreamHandler(deploy_stderr)
    handler.setFormatter(formatter)
    handler.setLevel('WARN')
    log.addHandler(handler)

    return log, deploy_stdout, deploy_stderr


def wait_required_images(client, log, images_timeout, poll_period, images):
    log.info(
        'Waiting for images: %s' % ', '.join(images))
    timeout = time.time() + images_timeout

    def image_prefixes(images):
        for image in images:
            if ':' in image:
                yield image
            else:
                yield '%s:' % image

    matching_prefixes = list(image_prefixes(images))

    def image_names(all_images):
        for image in all_images:
            for name in image['RepoTags']:
                yield name

    while matching_prefixes:
        all_images = list(image_names(client.images()))
        for image_prefix in matching_prefixes:
            for image in all_images:
                if image.startswith(image_prefix):
                    log.info('Found image: %s' % image)
                    matching_prefixes.remove(image_prefix)

        if time.time() > timeout:
            raise Exception('Timed out after %s seconds waiting for '
                            'matching images: %s' % (
                                images_timeout,
                                ', '.join(matching_prefixes)))
        if poll_period:
            time.sleep(poll_period)


def wait_required_containers(client, log,
                             containers_timeout, poll_period,
                             container_patterns):
    patterns = container_patterns.values()
    log.info(
        'Waiting for containers matching: %s' % ', '.join(patterns))

    timeout = time.time() + containers_timeout

    def containers_names(containers):
        for container in containers:
            for name in container['Names']:
                yield name

    waiting_for = dict((v, re.compile(v)) for v in patterns)
    while waiting_for:
        for name in containers_names(client.containers()):
            for k, v in six.iteritems(waiting_for):
                if v.match(name):
                    log.info('Pattern %s matches: %s' % (k, name))
                    del(waiting_for[k])
                    break
        if time.time() > timeout:
            raise Exception('Timed out after %s seconds waiting for '
                            'matching containers: %s' % (
                                containers_timeout,
                                ', '.join(waiting_for.keys)))
        if poll_period:
            time.sleep(poll_period)


def main(argv=sys.argv, sys_stdin=sys.stdin, sys_stdout=sys.stdout):
    (log, deploy_stdout, deploy_stderr) = configure_logging()
    client = get_client(log)

    c = json.load(sys.stdin)

    images_timeout = c['options'].get(
        'images_timeout', DEFAULT_IMAGES_TIMEOUT)
    containers_timeout = c['options'].get(
        'containers_timeout', DEFAULT_CONTAINERS_TIMEOUT)
    poll_period = c['options'].get(
        'poll_period', DEFAULT_POLL_PERIOD)

    pod_state = 0

    try:
        wait_required_images(
            client,
            log,
            images_timeout,
            poll_period,
            required_images(c))

        wait_required_containers(
            client,
            log,
            containers_timeout,
            poll_period,
            required_container_patterns(c))

    except Exception as ex:
        pod_state = 1
        log.error('An error occurred deploying pod %s' % c['id'])
        log.exception(ex)

    response = {
        'deploy_stdout': deploy_stdout.getvalue(),
        'deploy_stderr': deploy_stderr.getvalue(),
        'deploy_status_code': pod_state,
    }
    json.dump(response, sys_stdout)


if __name__ == '__main__':
    sys.exit(main(sys.argv))



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import fixtures
import json
import logging
import os
import tempfile
import yaml

from tests import common

log = logging.getLogger('test_hook_apply_config')


class HookApplyConfigTest(common.RunScriptTest):

    data = {
        'id': 'test_apply_config',
        'name': 'fake_resource_name',
        'group': 'apply-config',
        'config': {'foo': 'bar'}
    }

    def setUp(self):
        super(HookApplyConfigTest, self).setUp()
        self.hook_path = self.relative_path(
            __file__,
            '..',
            'heat-config-apply-config/install.d/hook-apply-config.py')

        self.metadata_dir = self.useFixture(fixtures.TempDir())
        self.templates_dir = self.useFixture(fixtures.TempDir())
        tmp_dir = tempfile.NamedTemporaryFile(mode='w', delete=False).name
        os.unlink(tmp_dir)
        self.tmp_file = os.path.basename(tmp_dir)
        self.out_dir = self.templates_dir.join('tmp')

        self.metadata = self.metadata_dir.join(self.tmp_file)

        self.env = os.environ.copy()
        self.env.update({
            'OS_CONFIG_FILES': self.metadata,
            'OS_CONFIG_APPLIER_TEMPLATES': self.templates_dir.join(),
        })

        # our fake metadata file
        with open(self.metadata, "w+") as md:
            md.write(json.dumps({'foo': 'bar'}))

        # This is our fake template root we use to verify os-apply-config
        # works as expected
        os.mkdir(self.out_dir)
        with open(os.path.join(self.out_dir, self.tmp_file), "w+") as template:
            template.write("foo={{foo}}")

    def test_hook(self):

        returncode, stdout, stderr = self.run_cmd(
            [self.hook_path], self.env, json.dumps(self.data))

        self.assertEqual(0, returncode, stderr)
        ret = yaml.safe_load(stdout)
        self.assertIsNotNone(ret['deploy_stderr'])
        self.assertEqual('', ret['deploy_stdout'])
        self.assertEqual(0, ret['deploy_status_code'])
        f = os.path.join('/tmp', self.tmp_file)
        with open(f) as out_file:
            self.assertEqual('foo=bar', out_file.read())
        os.unlink(f)



#!/usr/bin/env python
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
'''
A fake heat-config hook for unit testing the 55-heat-config
os-refresh-config script.
'''

import json
import os
import sys


def main(argv=sys.argv):
    c = json.load(sys.stdin)

    inputs = {}
    for input in c['inputs']:
        inputs[input['name']] = input.get('value', '')

    response = {}

    # populate outputs from inputs of the same name
    for output in c.get('outputs') or []:
        output_name = output['name']
        response[output_name] = inputs.get(output_name, '')

    # populate deploy outputs from the inputs of the same name
    response.update({
        'deploy_stdout': inputs.get('deploy_stdout', 'stdout'),
        'deploy_stderr': inputs.get('deploy_stderr', 'stderr'),
        'deploy_status_code': inputs.get('deploy_status_code', '0'),
    })

    # write out stdin and stdout json for test asserts
    stdin_path = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), '%s.stdin' % c['group'])
    stdout_path = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), '%s.stdout' % c['group'])

    with open(stdin_path, 'w') as f:
        json.dump(c, f)
        f.flush()
    with open(stdout_path, 'w') as f:
        json.dump(response, f)
        f.flush()
    json.dump(response, sys.stdout)

if __name__ == '__main__':
    sys.exit(main(sys.argv))



#!/usr/bin/env python
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import logging
import os
import sys

import requests

from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry

try:
    from heatclient import client as heatclient
except ImportError:
    heatclient = None

try:
    from keystoneclient.v3 import client as ksclient
except ImportError:
    ksclient = None

try:
    from zaqarclient.queues.v1 import client as zaqarclient
except ImportError:
    zaqarclient = None


MAX_RESPONSE_SIZE = 950000


def init_logging():
    log = logging.getLogger('heat-config-notify')
    handler = logging.StreamHandler(sys.stderr)
    handler.setFormatter(
        logging.Formatter(
            '[%(asctime)s] (%(name)s) [%(levelname)s] %(message)s'))
    log.addHandler(handler)
    log.setLevel('DEBUG')
    return log


def trim_response(response, trimmed_values=None):
    """Trim selected values from response.

    Makes given response smaller or the same size as MAX_RESPONSE_SIZE by
    trimming given trimmed_values from response dict from the left side
    (beginning). Returns trimmed and serialized JSON response itself.
    """

    trimmed_values = trimmed_values or ('deploy_stdout', 'deploy_stderr')
    str_response = json.dumps(response, ensure_ascii=True)
    len_total = len(str_response)
    offset = MAX_RESPONSE_SIZE - len_total
    if offset >= 0:
        return str_response
    offset = abs(offset)
    for key in trimmed_values:
        len_value = len(response[key])
        cut = int(round(float(len_value) / len_total * offset))
        response[key] = response[key][cut:]
    str_response = json.dumps(response, ensure_ascii=True, encoding='utf-8')
    return str_response


def main(argv=sys.argv, stdin=sys.stdin):

    log = init_logging()
    usage = ('Usage:\n  heat-config-notify /path/to/config.json '
             '< /path/to/signal_data.json')

    if len(argv) < 2:
        log.error(usage)
        return 1

    try:
        signal_data = json.load(stdin)
    except ValueError:
        log.warn('No valid json found on stdin')
        signal_data = {}

    conf_file = argv[1]
    if not os.path.exists(conf_file):
        log.error('No config file %s' % conf_file)
        log.error(usage)
        return 1

    c = json.load(open(conf_file))

    iv = dict((i['name'], i['value']) for i in c['inputs'])

    if 'deploy_signal_id' in iv:
        sigurl = iv.get('deploy_signal_id')
        sigverb = iv.get('deploy_signal_verb', 'POST')
        log.debug('Signaling to %s via %s' % (sigurl, sigverb))
        # we need to trim log content because Heat response size is limited
        # by max_json_body_size = 1048576
        str_signal_data = trim_response(signal_data)
        session = requests.Session()
        # Retry if connection issues occur or the service is returning a 5xx
        retry = Retry(
            total=10,
            read=10,
            connect=10,
            backoff_factor=0.5,
            status_forcelist=(500, 502, 503, 504)
        )
        adapter = HTTPAdapter(max_retries=retry)
        session.mount('http://', adapter)
        session.mount('https://', adapter)

        if sigverb == 'PUT':
            r = session.put(sigurl, data=str_signal_data,
                            headers={'content-type': 'application/json'})
        else:
            r = session.post(sigurl, data=str_signal_data,
                             headers={'content-type': 'application/json'})
        log.debug('Response %s ' % r)

    if 'deploy_queue_id' in iv:
        queue_id = iv.get('deploy_queue_id')
        log.debug('Signaling to queue %s' % (queue_id,))

        ks = ksclient.Client(
            auth_url=iv['deploy_auth_url'],
            user_id=iv['deploy_user_id'],
            password=iv['deploy_password'],
            project_id=iv['deploy_project_id'])
        endpoint = ks.service_catalog.url_for(
            service_type='messaging', endpoint_type='publicURL',
            region_name=iv.get('deploy_region_name'))

        conf = {
            'auth_opts': {
                'backend': 'keystone',
                'options': {
                    'os_auth_token': ks.auth_token,
                    'os_project_id': iv['deploy_project_id'],
                }
            }
        }
        cli = zaqarclient.Client(endpoint, conf=conf, version=1.1)
        queue = cli.queue(queue_id)
        r = queue.post({'body': signal_data, 'ttl': 600})
        log.debug('Response %s ' % r)

    elif 'deploy_auth_url' in iv:
        ks = ksclient.Client(
            auth_url=iv['deploy_auth_url'],
            user_id=iv['deploy_user_id'],
            password=iv['deploy_password'],
            project_id=iv['deploy_project_id'])
        endpoint = ks.service_catalog.url_for(
            service_type='orchestration', endpoint_type='publicURL',
            region_name=iv.get('deploy_region_name'))
        log.debug('Signalling to %s' % endpoint)
        heat = heatclient.Client(
            '1', endpoint, token=ks.auth_token)
        r = heat.resources.signal(
            iv.get('deploy_stack_id'),
            iv.get('deploy_resource_name'),
            data=signal_data)
        log.debug('Response %s ' % r)

    return 0


if __name__ == '__main__':
    sys.exit(main(sys.argv, sys.stdin))



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import os
import subprocess

import testtools


class RunScriptTest(testtools.TestCase):

    def relative_path(self, from_path, *to_paths):
        return os.path.join(
            os.path.dirname(os.path.realpath(from_path)), *to_paths)

    def run_cmd(self, args, env, input_str=None):
        subproc = subprocess.Popen(args,
                                   stdin=subprocess.PIPE,
                                   stdout=subprocess.PIPE,
                                   stderr=subprocess.PIPE,
                                   env=env)
        if input_str:
            input_str = input_str.encode('utf-8')
        stdout, stderr = subproc.communicate(input=input_str)
        return subproc.returncode, stdout, stderr

    def json_from_file(self, path):
        with open(path) as f:
            return json.load(f)

    def json_from_files(self, path, count, delete_after=True):
        for i in range(count + 1):
            if i == 0:
                filename = path
            else:
                filename = '%s_%d' % (path, i)

            # check there are not more files than the exact number requested
            if i == count:
                self.assertFalse(
                    os.path.isfile(filename),
                    'More than %d invocations' % count
                )
            else:
                with open(filename) as f:
                    yield json.load(f)
                if delete_after:
                    os.remove(filename)



#!/usr/bin/env python
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import logging
import os
import subprocess
import sys


HIERA_DATADIR = os.environ.get('HEAT_PUPPET_HIERA_DATADIR',
                               '/etc/puppet/hieradata')
HIERA_CONFIG = os.environ.get('HEAT_HIERA_CONFIG', '/etc/puppet/hiera.yaml')
HIERA_ELEMENT_CHECK_CMD = os.environ.get('HEAT_HIERA_ELEMENT_CHECK_CMD',
                                         'os-apply-config '
                                         '--key hiera.datafiles '
                                         '--type raw --key-default empty')


HIERA_CONFIG_BASE = """
---
:backends:
  - json
:json:
  :datadir: %(datadir)s
:hierarchy:
""" % {'datadir': HIERA_DATADIR}


def prepare_dir(path):
    if not os.path.isdir(path):
        os.makedirs(path, 0o700)


def exit_legacy_hiera_detected():
    try:
        subproc = subprocess.Popen(HIERA_ELEMENT_CHECK_CMD.split(" "),
                                   stdout=subprocess.PIPE,
                                   stderr=subprocess.PIPE)
        stdout, stderr = subproc.communicate()
        rs_stdout = stdout.rstrip().decode('utf-8', 'replace')
        if rs_stdout != 'empty':
            err_msg = ('Legacy hieradata from os-apply-config has been '
                       'detected - %s. Please update all of your interfaces '
                       'to use the new heat-agents hiera hook before '
                       'proceeding' % rs_stdout)
            response = {
                'deploy_stdout': stdout,
                'deploy_stderr': err_msg,
                'deploy_status_code': 1,
            }

            json.dump(response, sys.stdout)
            sys.exit(0)

    except OSError:
        # os-apply-config is not installed? Assume there is no legacy data.
        pass


def main(argv=sys.argv):
    log = logging.getLogger('heat-config')
    handler = logging.StreamHandler(sys.stderr)
    handler.setFormatter(
        logging.Formatter(
            '[%(asctime)s] (%(name)s) [%(levelname)s] %(message)s'))
    log.addHandler(handler)
    log.setLevel('DEBUG')

    c = json.load(sys.stdin)['config']
    exit_legacy_hiera_detected()

    prepare_dir(HIERA_DATADIR)

    hiera_config_file = os.path.join(HIERA_CONFIG)

    # allow the end user to order the hiera config as they wish
    if 'hierarchy' in c:
        with os.fdopen(os.open(hiera_config_file,
                               os.O_CREAT | os.O_TRUNC | os.O_WRONLY, 0o600),
                       'w') as config_file:
            config_file.write(HIERA_CONFIG_BASE)
            for item in c['hierarchy']:
                config_file.write('  - %s\n' % item)

    # write out the datafiles as YAML
    if 'datafiles' in c:
        for name, data in c['datafiles'].items():
            hiera_data = os.path.join(HIERA_DATADIR, '%s.json' % name)
            with os.fdopen(os.open(hiera_data,
                                   os.O_CREAT | os.O_TRUNC | os.O_WRONLY,
                                   0o600),
                           'w') as hiera_data_file:
                json.dump(data, hiera_data_file, indent=4, sort_keys=True,
                          separators=(',', ': '))

    response = {
        'deploy_stdout': '',
        'deploy_stderr': '',
        'deploy_status_code': 0,
    }

    json.dump(response, sys.stdout)


if __name__ == '__main__':
    sys.exit(main(sys.argv))



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy
import json
import os
import tempfile

import fixtures
import six

from tests import common


class HookDockerCmdTest(common.RunScriptTest):
    data = {
        "name": "abcdef001",
        "group": "docker-cmd",
        "id": "abc123",
        "inputs": [{
            "name": "deploy_stack_id",
            "value": "the_stack",
        }, {
            "name": "deploy_resource_name",
            "value": "the_deployment",
        }],
        "config": {
            "db": {
                "name": "x",
                "image": "xxx",
                "privileged": False,
                "environment": ["foo=bar"],
                "env_file": "env.file",
                "start_order": 0
            },
            "web-ls": {
                "action": "exec",
                "start_order": 2,
                "command": ["web", "/bin/ls", "-l"]
            },
            "web": {
                "name": "y",
                "start_order": 1,
                "image": "yyy",
                "net": "host",
                "restart": "always",
                "privileged": True,
                "user": "root",
                "command": "/bin/webserver start",
                "volumes": [
                    "/run:/run",
                    "db:/var/lib/db"
                ],
                "environment": [
                    "KOLLA_CONFIG_STRATEGY=COPY_ALWAYS",
                    "FOO=BAR"
                ],
                "env_file": [
                    "foo.env",
                    "bar.conf"
                ]
            }
        }
    }

    data_exit_code = {
        "name": "abcdef001",
        "group": "docker-cmd",
        "id": "abc123",
        "config": {
            "web-ls": {
                "action": "exec",
                "command": ["web", "/bin/ls", "-l"],
                "exit_codes": [0, 1]
            }
        }
    }

    def setUp(self):
        super(HookDockerCmdTest, self).setUp()
        self.hook_path = self.relative_path(
            __file__,
            '..',
            'heat-config-docker-cmd/install.d/hook-docker-cmd.py')

        self.cleanup_path = self.relative_path(
            __file__,
            '..',
            'heat-config-docker-cmd/',
            'os-refresh-config/configure.d/50-heat-config-docker-cmd')

        self.fake_tool_path = six.text_type(self.relative_path(
            __file__,
            'config-tool-fake.py'))

        self.working_dir = self.useFixture(fixtures.TempDir())
        self.outputs_dir = self.useFixture(fixtures.TempDir())
        self.test_state_path = self.outputs_dir.join('test_state.json')

        self.env = os.environ.copy()
        self.env.update({
            'HEAT_DOCKER_CMD': self.fake_tool_path,
            'TEST_STATE_PATH': self.test_state_path,
        })

    def check_basic_response(self, state):
        self.assertEqual([
            self.fake_tool_path,
            u'inspect',
            u'--type',
            u'image',
            u'--format',
            u'exists',
            u'xxx'
        ], state[0]['args'])
        self.assertEqual([
            self.fake_tool_path,
            u'pull',
            u'xxx'
        ], state[1]['args'])
        self.assertEqual([
            self.fake_tool_path,
            u'inspect',
            u'--type',
            u'image',
            u'--format',
            u'exists',
            u'yyy'
        ], state[2]['args'])
        self.assertEqual([
            self.fake_tool_path,
            u'pull',
            u'yyy'
        ], state[3]['args'])
        self.assertEqual([
            self.fake_tool_path,
            u'ps',
            u'-a',
            u'--filter',
            u'label=managed_by=docker-cmd',
            u'--filter',
            u'label=config_id=abc123',
            u'--format',
            u'{{.Names}} {{.Label "container_name"}}'
        ], state[4]['args'])
        self.assertEqual([
            self.fake_tool_path,
            u'ps',
            u'-a',
            u'--filter',
            u'label=managed_by=docker-cmd',
            u'--format',
            u'{{.Names}} {{.Label "container_name"}}'
        ], state[5]['args'])
        self.assertEqual([
            self.fake_tool_path,
            u'ps',
            u'-a',
            u'--filter',
            u'label=managed_by=docker-cmd',
            u'--filter',
            u'label=config_id=abc123',
            u'--format',
            u'{{.Names}} {{.Label "container_name"}}'
        ], state[6]['args'])
        self.assertEqual([
            self.fake_tool_path,
            u'inspect',
            u'--type',
            u'container',
            u'--format',
            u'exists',
            u'db'
        ], state[7]['args'])

    def assert_args_and_labels(self, expected_args, expected_labels, observed):
        '''Assert the labels arguments separately to other arguments.

        Tests that each expected_labels label exists, and remaining
        expected arguments match exactly.

        This allows paunch to add new label arguments without breaking these
        tests.
        '''

        args = []
        labels = []
        j = 0
        while j < len(observed):
            if observed[j] == '--label':
                j += 1
                labels.append(observed[j])
            else:
                args.append(observed[j])
            j += 1

        self.assertEqual(expected_args, args)
        for label in expected_labels:
            self.assertIn(label, labels)

    def test_hook(self):

        self.env.update({
            'TEST_RESPONSE': json.dumps([
                # inspect for image xxx
                {},
                # poll for image xxx
                {},
                # inspect for image yyy
                {},
                # poll for image yyy
                {},
                # ps for delete missing
                {},
                # ps for renames
                {},
                # ps for currently running containers
                {},
                # inspect for db unique container name
                {},
                # docker run db
                {'stderr': 'Creating db...'},
                # inspect for web unique container name
                {},
                # docker run web
                {'stderr': 'Creating web...'},
                # name lookup for exec web
                {'stdout': 'web'},
                # docker exec web
                {'stderr': 'one.txt\ntwo.txt\nthree.txt'},
            ])
        })
        returncode, stdout, stderr = self.run_cmd(
            [self.hook_path], self.env, json.dumps(self.data))

        self.assertEqual(0, returncode, stderr)

        self.assertEqual({
            'deploy_stdout': '',
            'deploy_stderr': 'Creating db...\n'
                             'Creating web...\n'
                             'one.txt\ntwo.txt\nthree.txt',
            'deploy_status_code': 0
        }, json.loads(stdout.decode('utf-8')))

        state = list(self.json_from_files(self.test_state_path, 13))
        self.check_basic_response(state)
        self.assert_args_and_labels([
            self.fake_tool_path,
            u'run',
            u'--name',
            u'db',
            u'--detach=true',
            u'--env-file=env.file',
            u'--env=foo=bar',
            u'--privileged=false',
            u'xxx'
            u''
        ], [
            u'deploy_stack_id=the_stack',
            u'deploy_resource_name=the_deployment',
            u'config_id=abc123',
            u'container_name=db',
            u'managed_by=docker-cmd',
        ], state[8]['args'])
        self.assertEqual([
            self.fake_tool_path,
            u'inspect',
            u'--type',
            u'container',
            u'--format',
            u'exists',
            u'web',
        ], state[9]['args'])
        self.assert_args_and_labels([
            self.fake_tool_path,
            u'run',
            u'--name',
            u'web',
            u'--detach=true',
            u'--env-file=foo.env',
            u'--env-file=bar.conf',
            u'--env=KOLLA_CONFIG_STRATEGY=COPY_ALWAYS',
            u'--env=FOO=BAR',
            u'--net=host',
            u'--privileged=true',
            u'--restart=always',
            u'--user=root',
            u'--volume=/run:/run',
            u'--volume=db:/var/lib/db',
            u'yyy',
            u'/bin/webserver',
            u'start'
        ], [
            u'deploy_stack_id=the_stack',
            u'deploy_resource_name=the_deployment',
            u'config_id=abc123',
            u'container_name=web',
            u'managed_by=docker-cmd',
        ], state[10]['args'])
        self.assertEqual([
            self.fake_tool_path,
            u'ps',
            u'-a',
            u'--filter',
            u'label=container_name=web',
            u'--filter',
            u'label=config_id=abc123',
            u'--format',
            u'{{.Names}}',
        ], state[11]['args'])
        self.assertEqual([
            self.fake_tool_path,
            u'exec',
            u'web',
            u'/bin/ls',
            u'-l'
        ], state[12]['args'])

    def test_hook_exit_codes(self):

        self.env.update({
            'TEST_RESPONSE': json.dumps([
                # ps for delete missing
                {},
                # ps for renames
                {},
                # ps for currently running containers
                {},
                {'stdout': 'web'},
                {
                    'stdout': '',
                    'stderr': 'Warning: custom exit code',
                    'returncode': 1
                }
            ])
        })
        returncode, stdout, stderr = self.run_cmd(
            [self.hook_path], self.env, json.dumps(self.data_exit_code))

        self.assertEqual({
            'deploy_stdout': '',
            'deploy_stderr': 'Warning: custom exit code',
            'deploy_status_code': 0
        }, json.loads(stdout.decode('utf-8')))

        state = list(self.json_from_files(self.test_state_path, 5))
        self.assertEqual([
            self.fake_tool_path,
            'ps',
            '-a',
            '--filter',
            'label=managed_by=docker-cmd',
            '--filter',
            'label=config_id=abc123',
            '--format',
            '{{.Names}} {{.Label "container_name"}}'
        ], state[0]['args'])
        self.assertEqual([
            self.fake_tool_path,
            'ps',
            '-a',
            '--filter',
            'label=managed_by=docker-cmd',
            '--format',
            '{{.Names}} {{.Label "container_name"}}'
        ], state[1]['args'])
        self.assertEqual([
            self.fake_tool_path,
            'ps',
            '-a',
            '--filter',
            'label=managed_by=docker-cmd',
            '--filter',
            'label=config_id=abc123',
            '--format',
            '{{.Names}} {{.Label "container_name"}}'
        ], state[2]['args'])
        self.assertEqual([
            self.fake_tool_path,
            'ps',
            '-a',
            '--filter',
            'label=container_name=web',
            '--filter',
            'label=config_id=abc123',
            '--format',
            '{{.Names}}',
        ], state[3]['args'])
        self.assertEqual([
            self.fake_tool_path,
            'exec',
            'web',
            '/bin/ls',
            '-l'
        ], state[4]['args'])

    def test_hook_failed(self):

        self.env.update({
            'TEST_RESPONSE': json.dumps([
                # inspect for image xxx
                {},
                # poll for image xxx
                {},
                # inspect for image yyy
                {},
                # poll for image yyy
                {},
                # ps for delete missing
                {},
                # ps for renames
                {},
                # ps for currently running containers
                {},
                # inspect for db unique container name
                {},
                # docker run db
                {'stderr': 'Creating db...'},
                # inspect for web unique container name
                {},
                # docker run web
                {'stderr': 'Creating web...'},
                # name lookup for exec web
                {'stdout': 'web'},
                # docker exec web fails
                {
                    'stdout': '',
                    'stderr': 'No such file or directory',
                    'returncode': 2
                }
            ])
        })
        returncode, stdout, stderr = self.run_cmd(
            [self.hook_path], self.env, json.dumps(self.data))

        self.assertEqual({
            'deploy_stdout': '',
            'deploy_stderr': 'Creating db...\n'
                             'Creating web...\n'
                             'No such file or directory',
            'deploy_status_code': 2
        }, json.loads(stdout.decode('utf-8')))

        state = list(self.json_from_files(self.test_state_path, 13))
        self.check_basic_response(state)
        self.assert_args_and_labels([
            self.fake_tool_path,
            u'run',
            u'--name',
            u'db',
            u'--detach=true',
            u'--env-file=env.file',
            u'--env=foo=bar',
            u'--privileged=false',
            u'xxx'
            u''
        ], [
            u'deploy_stack_id=the_stack',
            u'deploy_resource_name=the_deployment',
            u'config_id=abc123',
            u'container_name=db',
            u'managed_by=docker-cmd',
        ], state[8]['args'])
        self.assertEqual([
            self.fake_tool_path,
            u'inspect',
            u'--type',
            u'container',
            u'--format',
            u'exists',
            u'web',
        ], state[9]['args'])
        self.assert_args_and_labels([
            self.fake_tool_path,
            u'run',
            u'--name',
            u'web',
            u'--detach=true',
            u'--env-file=foo.env',
            u'--env-file=bar.conf',
            u'--env=KOLLA_CONFIG_STRATEGY=COPY_ALWAYS',
            u'--env=FOO=BAR',
            u'--net=host',
            u'--privileged=true',
            u'--restart=always',
            u'--user=root',
            u'--volume=/run:/run',
            u'--volume=db:/var/lib/db',
            u'yyy',
            u'/bin/webserver',
            u'start'
        ], [
            u'deploy_stack_id=the_stack',
            u'deploy_resource_name=the_deployment',
            u'config_id=abc123',
            u'container_name=web',
            u'managed_by=docker-cmd',
        ], state[10]['args'])
        self.assertEqual([
            self.fake_tool_path,
            u'ps',
            u'-a',
            u'--filter',
            u'label=container_name=web',
            u'--filter',
            u'label=config_id=abc123',
            u'--format',
            u'{{.Names}}',
        ], state[11]['args'])
        self.assertEqual([
            self.fake_tool_path,
            u'exec',
            u'web',
            u'/bin/ls',
            u'-l'
        ], state[12]['args'])

    def test_hook_unique_names(self):
        self.env.update({
            'TEST_RESPONSE': json.dumps([
                # inspect for image xxx
                {},
                # poll for image xxx
                {},
                # inspect for image yyy
                {},
                # poll for image yyy
                {},
                # ps for delete missing in this config id
                {},
                # ps for renames
                {'stdout': 'web web\ndb db\n'},
                # ps for currently running containers in this config id
                {},
                # inspect for db unique container name
                {'stdout': 'exists'},
                {
                    'stderr': 'Error: No such container: db-blah',
                    'returncode': 1
                },
                # docker run db
                {'stderr': 'Creating db...'},
                # # inspect for web unique container name
                {'stdout': 'exists'},
                {
                    'stderr': 'Error: No such container: web-blah',
                    'returncode': 1
                },
                # # docker run web
                {'stderr': 'Creating web...'},
                # name lookup for exec web
                {'stdout': 'web-asdf1234'},
                # docker exec web-asdf1234
                {'stderr': 'one.txt\ntwo.txt\nthree.txt'},
            ])
        })

        returncode, stdout, stderr = self.run_cmd(
            [self.hook_path], self.env, json.dumps(self.data))

        self.assertEqual(0, returncode, stderr)

        state = list(self.json_from_files(self.test_state_path, 15))
        dd = []
        for i in state:
            dd.append(i['args'])

        db_container_name = state[8]['args'][6]
        web_container_name = state[11]['args'][6]
        self.assertRegex(db_container_name, 'db-[0-9a-z]{8}')
        self.assertRegex(web_container_name, 'web-[0-9a-z]{8}')
        self.check_basic_response(state)
        self.assertEqual([
            self.fake_tool_path,
            u'inspect',
            u'--type',
            u'container',
            u'--format',
            u'exists',
            db_container_name,
        ], state[8]['args'])
        self.assert_args_and_labels([
            self.fake_tool_path,
            u'run',
            u'--name',
            db_container_name,
            u'--detach=true',
            u'--env-file=env.file',
            u'--env=foo=bar',
            u'--privileged=false',
            u'xxx'
        ], [
            u'deploy_stack_id=the_stack',
            u'deploy_resource_name=the_deployment',
            u'config_id=abc123',
            u'container_name=db',
            u'managed_by=docker-cmd',
        ], state[9]['args'])
        self.assertEqual([
            self.fake_tool_path,
            u'inspect',
            u'--type',
            u'container',
            u'--format',
            u'exists',
            u'web',
        ], state[10]['args'])
        self.assertEqual([
            self.fake_tool_path,
            u'inspect',
            u'--type',
            u'container',
            u'--format',
            u'exists',
            web_container_name,
        ], state[11]['args'])
        self.assert_args_and_labels([
            self.fake_tool_path,
            u'run',
            u'--name',
            web_container_name,
            u'--detach=true',
            u'--env-file=foo.env',
            u'--env-file=bar.conf',
            u'--env=KOLLA_CONFIG_STRATEGY=COPY_ALWAYS',
            u'--env=FOO=BAR',
            u'--net=host',
            u'--privileged=true',
            u'--restart=always',
            u'--user=root',
            u'--volume=/run:/run',
            u'--volume=db:/var/lib/db',
            u'yyy',
            u'/bin/webserver',
            u'start'
        ], [
            u'deploy_stack_id=the_stack',
            u'deploy_resource_name=the_deployment',
            u'config_id=abc123',
            u'container_name=web',
            u'managed_by=docker-cmd',
        ], state[12]['args'])
        self.assertEqual([
            self.fake_tool_path,
            u'ps',
            u'-a',
            u'--filter',
            u'label=container_name=web',
            u'--filter',
            u'label=config_id=abc123',
            u'--format',
            u'{{.Names}}',
        ], state[13]['args'])
        self.assertEqual([
            self.fake_tool_path,
            u'exec',
            u'web-asdf1234',
            u'/bin/ls',
            u'-l'
        ], state[14]['args'])

    def test_cleanup_deleted(self):
        self.env.update({
            'TEST_RESPONSE': json.dumps([{
                # first run, no running containers
                'stdout': '\n'
            }, {
                # list name and container_name label for all containers
                'stdout': '\n'
            }])
        })
        conf_dir = self.useFixture(fixtures.TempDir()).join()
        with tempfile.NamedTemporaryFile(dir=conf_dir, delete=False,
                                         mode='w') as f:
            f.write(json.dumps([self.data]))
            f.flush()
            self.env['HEAT_SHELL_CONFIG'] = f.name

            returncode, stdout, stderr = self.run_cmd(
                [self.cleanup_path], self.env)

        # on the first run, no docker rm calls made
        state = list(self.json_from_files(self.test_state_path, 2))
        self.assertEqual([
            self.fake_tool_path,
            'ps',
            '-a',
            '--filter',
            'label=managed_by=docker-cmd',
            '--format',
            '{{.Label "config_id"}}'
        ], state[0]['args'])
        self.assertEqual([
            self.fake_tool_path,
            'ps',
            '-a',
            '--filter',
            'label=managed_by=docker-cmd',
            '--format',
            '{{.Names}} {{.Label "container_name"}}'
        ], state[1]['args'])

        self.env.update({
            'TEST_RESPONSE': json.dumps([{
                # list config_id labels, 3 containers same config
                'stdout': 'abc123\nabc123\nabc123\n'
            }, {
                # list containers with config_id
                'stdout': '111\n222\n333\n'
            }, {
                'stdout': '111 deleted'
            }, {
                'stdout': '222 deleted'
            }, {
                'stdout': '333 deleted'
            }, {
                # list name and container_name label for all containers
                'stdout': '\n'
            }])
        })

        # run again with empty config data
        with tempfile.NamedTemporaryFile(dir=conf_dir, delete=False,
                                         mode='w') as f:
            f.write(json.dumps([]))
            f.flush()
            self.env['HEAT_SHELL_CONFIG'] = f.name

            returncode, stdout, stderr = self.run_cmd(
                [self.cleanup_path], self.env)

        # on the second run, abc123 is deleted,
        # docker rm is run on all containers
        state = list(self.json_from_files(self.test_state_path, 6))
        self.assertEqual([
            self.fake_tool_path,
            'ps',
            '-a',
            '--filter',
            'label=managed_by=docker-cmd',
            '--format',
            '{{.Label "config_id"}}'
        ], state[0]['args'])
        self.assertEqual([
            self.fake_tool_path,
            'ps',
            '-q',
            '-a',
            '--filter',
            'label=managed_by=docker-cmd',
            '--filter',
            'label=config_id=abc123'
        ], state[1]['args'])
        self.assertEqual([
            self.fake_tool_path,
            'rm',
            '-f',
            '111',
        ], state[2]['args'])
        self.assertEqual([
            self.fake_tool_path,
            'rm',
            '-f',
            '222',
        ], state[3]['args'])
        self.assertEqual([
            self.fake_tool_path,
            'rm',
            '-f',
            '333',
        ], state[4]['args'])
        self.assertEqual([
            self.fake_tool_path,
            'ps',
            '-a',
            '--filter',
            'label=managed_by=docker-cmd',
            '--format',
            '{{.Names}} {{.Label "container_name"}}'
        ], state[5]['args'])

    def test_cleanup_changed(self):
        self.env.update({
            'TEST_RESPONSE': json.dumps([{
                # list config_id labels, 3 containers same config
                'stdout': 'abc123\nabc123\nabc123\n'
            }, {
                # list name and container_name label for all containers
                'stdout': '111 111\n'
                          '222 222\n'
                          '333\n'
            }])
        })
        conf_dir = self.useFixture(fixtures.TempDir()).join()
        with tempfile.NamedTemporaryFile(dir=conf_dir, delete=False,
                                         mode='w') as f:
            f.write(json.dumps([self.data]))
            f.flush()
            self.env['HEAT_SHELL_CONFIG'] = f.name

            returncode, stdout, stderr = self.run_cmd(
                [self.cleanup_path], self.env)

        # on the first run, no docker rm calls made
        state = list(self.json_from_files(self.test_state_path, 2))
        self.assertEqual([
            self.fake_tool_path,
            'ps',
            '-a',
            '--filter',
            'label=managed_by=docker-cmd',
            '--format',
            '{{.Label "config_id"}}'
        ], state[0]['args'])
        self.assertEqual([
            self.fake_tool_path,
            'ps',
            '-a',
            '--filter',
            'label=managed_by=docker-cmd',
            '--format',
            '{{.Names}} {{.Label "container_name"}}'
        ], state[1]['args'])

        # run again with changed config data
        self.env.update({
            'TEST_RESPONSE': json.dumps([{
                # list config_id labels, 3 containers same config
                'stdout': 'abc123\nabc123\nabc123\n'
            }, {
                # list containers with config_id
                'stdout': '111\n222\n333\n'
            }, {
                'stdout': '111 deleted'
            }, {
                'stdout': '222 deleted'
            }, {
                'stdout': '333 deleted'
            }, {
                # list name and container_name label for all containers
                'stdout': 'abc123 abc123\n'
            }])
        })
        new_data = copy.deepcopy(self.data)
        new_data['config']['web']['image'] = 'yyy'
        new_data['id'] = 'def456'
        with tempfile.NamedTemporaryFile(dir=conf_dir, delete=False,
                                         mode='w') as f:
            f.write(json.dumps([new_data]))
            f.flush()
            self.env['HEAT_SHELL_CONFIG'] = f.name

            returncode, stdout, stderr = self.run_cmd(
                [self.cleanup_path], self.env)

        # on the second run, abc123 is deleted,
        # docker rm is run on all containers
        state = list(self.json_from_files(self.test_state_path, 6))
        self.assertEqual([
            self.fake_tool_path,
            'ps',
            '-a',
            '--filter',
            'label=managed_by=docker-cmd',
            '--format',
            '{{.Label "config_id"}}'
        ], state[0]['args'])
        self.assertEqual([
            self.fake_tool_path,
            'ps',
            '-q',
            '-a',
            '--filter',
            'label=managed_by=docker-cmd',
            '--filter',
            'label=config_id=abc123'
        ], state[1]['args'])
        self.assertEqual([
            self.fake_tool_path,
            'rm',
            '-f',
            '111',
        ], state[2]['args'])
        self.assertEqual([
            self.fake_tool_path,
            'rm',
            '-f',
            '222',
        ], state[3]['args'])
        self.assertEqual([
            self.fake_tool_path,
            'rm',
            '-f',
            '333',
        ], state[4]['args'])
        self.assertEqual([
            self.fake_tool_path,
            'ps',
            '-a',
            '--filter',
            'label=managed_by=docker-cmd',
            '--format',
            '{{.Names}} {{.Label "container_name"}}'
        ], state[5]['args'])

    def test_cleanup_rename(self):
        self.env.update({
            'TEST_RESPONSE': json.dumps([{
                # list config_id labels, 3 containers same config
                'stdout': 'abc123\nabc123\nabc123\n'
            }, {
                # list name and container_name label for all containers
                'stdout': '111 111-s84nf83h\n'
                          '222 222\n'
                          '333 333-3nd83nfi\n'
            }])
        })
        conf_dir = self.useFixture(fixtures.TempDir()).join()
        with tempfile.NamedTemporaryFile(dir=conf_dir, delete=False,
                                         mode='w') as f:
            f.write(json.dumps([self.data]))
            f.flush()
            self.env['HEAT_SHELL_CONFIG'] = f.name

            returncode, stdout, stderr = self.run_cmd(
                [self.cleanup_path], self.env)

        # on the first run, no docker rm calls made
        state = list(self.json_from_files(self.test_state_path, 4))
        self.assertEqual([
            self.fake_tool_path,
            'ps',
            '-a',
            '--filter',
            'label=managed_by=docker-cmd',
            '--format',
            '{{.Label "config_id"}}'
        ], state[0]['args'])
        self.assertEqual([
            self.fake_tool_path,
            'ps',
            '-a',
            '--filter',
            'label=managed_by=docker-cmd',
            '--format',
            '{{.Names}} {{.Label "container_name"}}'
        ], state[1]['args'])
        self.assertEqual([
            self.fake_tool_path,
            'rename',
            '111',
            '111-s84nf83h'
        ], state[2]['args'])
        self.assertEqual([
            self.fake_tool_path,
            'rename',
            '333',
            '333-3nd83nfi'
        ], state[3]['args'])



#!/usr/bin/env python
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ast
import dpath
import json
import logging
import os
import six
import subprocess
import sys
import yaml


WORKING_DIR = os.environ.get('HEAT_DOCKER_COMPOSE_WORKING',
                             '/var/lib/heat-config/heat-config-docker-compose')

DOCKER_COMPOSE_CMD = os.environ.get('HEAT_DOCKER_COMPOSE_CMD',
                                    'docker-compose')


def prepare_dir(path):
    if not os.path.isdir(path):
        os.makedirs(path, 0o700)


def write_input_file(file_path, content):
    prepare_dir(os.path.dirname(file_path))
    with os.fdopen(os.open(
            file_path, os.O_CREAT | os.O_WRONLY, 0o600), 'w') as f:
        f.write(content)


def build_response(deploy_stdout, deploy_stderr, deploy_status_code):
    return {
        'deploy_stdout': deploy_stdout.decode('utf-8', 'replace'),
        'deploy_stderr': deploy_stderr.decode('utf-8', 'replace'),
        'deploy_status_code': deploy_status_code,
    }


def main(argv=sys.argv):
    log = logging.getLogger('heat-config')
    handler = logging.StreamHandler(sys.stderr)
    handler.setFormatter(
        logging.Formatter(
            '[%(asctime)s] (%(name)s) [%(levelname)s] %(message)s'))
    log.addHandler(handler)
    log.setLevel('DEBUG')

    c = json.load(sys.stdin)

    input_values = dict((i['name'], i['value']) for i in c['inputs'])

    proj = os.path.join(WORKING_DIR, c.get('name'))
    prepare_dir(proj)

    stdout, stderr = {}, {}

    if input_values.get('deploy_action') == 'DELETE':
        json.dump(build_response(stdout, stderr, 0), sys.stdout)
        return

    config = c.get('config', '')
    if not config:
        log.debug("No 'config' input found, nothing to do.")
        json.dump(build_response(stdout, stderr, 0), sys.stdout)
        return

    # convert config to dict
    if not isinstance(config, dict):
        config = ast.literal_eval(json.dumps(yaml.safe_load(config)))

    os.chdir(proj)

    compose_env_files = []
    for value in dpath.util.values(config, '*/env_file'):
        if isinstance(value, list):
            compose_env_files.extend(value)
        elif isinstance(value, six.string_types):
            compose_env_files.extend([value])

    input_env_files = {}
    if input_values.get('env_files'):
        input_env_files = dict(
            (i['file_name'], i['content'])
            for i in ast.literal_eval(input_values.get('env_files')))

    for file in compose_env_files:
        if file in input_env_files.keys():
            write_input_file(file, input_env_files.get(file))

    cmd = [
        DOCKER_COMPOSE_CMD,
        'up',
        '-d',
        '--no-build',
    ]

    log.debug('Running %s' % cmd)

    subproc = subprocess.Popen(cmd, stdout=subprocess.PIPE,
                               stderr=subprocess.PIPE)
    stdout, stderr = subproc.communicate()

    log.debug(stdout)
    log.debug(stderr)

    if subproc.returncode:
        log.error("Error running %s. [%s]\n" % (cmd, subproc.returncode))
    else:
        log.debug('Completed %s' % cmd)

    json.dump(build_response(stdout, stderr, subproc.returncode), sys.stdout)

if __name__ == '__main__':
    sys.exit(main(sys.argv))



# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


import json
import os

import fixtures

from tests import common


class HookAnsibleTest(common.RunScriptTest):

    data = {
        'id': '1234',
        'name': 'fake_resource_name',
        'group': 'ansible',
        'options': {},
        'inputs': [
            {'name': 'foo', 'value': 'bar'},
            {'name': 'another', 'value': 'input'}
        ],
        'config': 'the ansible playbook'
    }

    data_tags = {
        'id': '1234',
        'name': 'fake_resource_name_tags',
        'group': 'ansible',
        'options': {'tags': 'abc,def'},
        'inputs': [
            {'name': 'foo', 'value': 'bar'},
            {'name': 'another', 'value': 'input'}
        ],
        'config': 'the ansible playbook'
    }

    data_modulepath = data.copy()
    data_modulepath.update({
        'options': {'modulepath': '/opt/ansible:/usr/share/ansible'},
    })

    data_tags_modulepath = data.copy()
    data_tags_modulepath.update({
        'options': {'modulepath': '/opt/ansible:/usr/share/ansible',
                    'tags':       'abc,def'},
    })

    data_callback_plugins = data.copy()
    data_callback_plugins.update({
        'options': {'callback_plugins': '/usr/lib/python/foo'}
    })

    data_skip_tags = data.copy()
    data_skip_tags.update({'options': {'skip_tags': 'abc,def'}})

    data_skip_tags_empty = data.copy()
    data_skip_tags_empty.update({'options': {'skip_tags': ''}})

    data_tags_skip_tags = data.copy()
    data_tags_skip_tags.update({'options': {'tags': 'abc,def',
                                            'skip_tags': 'abc'}})

    data_skip_tags_modulepath = data.copy()
    data_skip_tags_modulepath.update({
        'options': {'modulepath': '/opt/ansible:/usr/share/ansible',
                    'skip_tags':  'abc,def'},
    })

    def setUp(self):
        super(HookAnsibleTest, self).setUp()
        self.hook_path = self.relative_path(
            __file__,
            '..',
            'heat-config-ansible/install.d/hook-ansible.py')

        self.fake_tool_path = self.relative_path(
            __file__,
            'config-tool-fake.py')

        self.working_dir = self.useFixture(fixtures.TempDir())
        self.outputs_dir = self.useFixture(fixtures.TempDir())
        self.test_state_path = self.outputs_dir.join('test_state.json')
        self.test_inventory = "localhost test_var=123,"

        self.env = os.environ.copy()
        self.env.update({
            'HEAT_ANSIBLE_WORKING': self.working_dir.join(),
            'HEAT_ANSIBLE_OUTPUTS': self.outputs_dir.join(),
            'HEAT_ANSIBLE_CMD': self.fake_tool_path,
            'TEST_STATE_PATH': self.test_state_path
        })

    def test_hook(self):
        self._hook_run()

    def test_hook_tags(self):
        self._hook_run(data=self.data_tags, options=['--tags', 'abc,def'])

    def test_hook_skip_tags(self):
        self._hook_run(data=self.data_skip_tags,
                       options=['--skip-tags', 'abc,def'])

    def test_hook_skip_tags_empty(self):
        self._hook_run(data=self.data_skip_tags_empty)

    def test_hook_tags_skip_tags(self):
        self._hook_run(data=self.data_tags_skip_tags,
                       options=['--skip-tags', 'abc',
                                '--tags', 'abc,def'])

    def test_hook_modulepath(self):
        self._hook_run(data=self.data_modulepath,
                       options=['--module-path',
                                '/opt/ansible:/usr/share/ansible'])

    def test_hook_tags_modulepath(self):
        self._hook_run(data=self.data_tags_modulepath,
                       options=['--module-path',
                                '/opt/ansible:/usr/share/ansible',
                                '--tags', 'abc,def'])

    def test_hook_skip_tags_modulepath(self):
        self._hook_run(data=self.data_skip_tags_modulepath,
                       options=['--module-path',
                                '/opt/ansible:/usr/share/ansible',
                                '--skip-tags', 'abc,def'])

    def test_hook_with_callback_plugins(self):
        self.assertTrue('ANSIBLE_CALLBACK_PLUGINS' not in self.env)

        state = self._hook_run(data=self.data_callback_plugins)
        opt = self.data_callback_plugins['options']['callback_plugins']
        self.assertEqual(self.env['ANSIBLE_CALLBACK_PLUGINS'], opt)
        self.assertEqual(state['env']['ANSIBLE_CALLBACK_PLUGINS'], opt)

    def _hook_run(self, data=None, options=None):
        self.env.update({
            'TEST_RESPONSE': json.dumps({
                'stdout': 'ansible success',
                'stderr': 'thing happened',
            }),
        })
        if data is not None and 'callback_plugins' in data['options']:
            self.env.update({
                'ANSIBLE_CALLBACK_PLUGINS': data['options']['callback_plugins']
            })
        returncode, stdout, stderr = self.run_cmd(
            [self.hook_path], self.env, json.dumps(data or self.data))

        self.assertEqual(0, returncode, stderr)
        self.assertEqual({
            'deploy_stdout': 'ansible success',
            'deploy_stderr': 'thing happened',
            'deploy_status_code': 0,
        }, json.loads(stdout.decode('utf-8')))

        state = self.json_from_file(self.test_state_path)
        ansible_playbook = self.working_dir.join('1234_playbook.yaml')
        vars_filename = self.working_dir.join('1234_variables.json')

        expected_args = [
            self.fake_tool_path,
            '-i',
            'localhost,']
        if options:
            expected_args += options
        expected_args += [
            ansible_playbook,
            '--extra-vars']
        expected_args.append('@%s' % vars_filename)
        self.assertEqual(expected_args, state['args'])

        # Write 'variables' to file
        variables = self.json_from_file(vars_filename)
        self.assertEqual('bar', variables['foo'])
        self.assertEqual('input', variables['another'])
        self.assertEqual('1234', variables['heat_config_id'])
        self.assertEqual(self.outputs_dir.join('1234'),
                         variables['heat_outputs_path'])

        # Write the executable 'config' to file
        with open(ansible_playbook) as f:
            self.assertEqual('the ansible playbook', f.read())

        return state

    def test_hook_inventory(self):

        self.env.update({
            'HEAT_ANSIBLE_INVENTORY': self.test_inventory,
            'TEST_RESPONSE': json.dumps({
                'stdout': 'ansible success',
                'stderr': 'thing happened',
            }),
        })
        returncode, stdout, stderr = self.run_cmd(
            [self.hook_path], self.env, json.dumps(self.data))

        self.assertEqual(0, returncode, stderr)
        self.assertEqual({
            'deploy_stdout': 'ansible success',
            'deploy_stderr': 'thing happened',
            'deploy_status_code': 0,
        }, json.loads(stdout.decode('utf-8')))

        state = self.json_from_file(self.test_state_path)
        ansible_playbook = self.working_dir.join('1234_playbook.yaml')
        vars_filename = self.working_dir.join('1234_variables.json')

        self.assertEqual(
            [
                self.fake_tool_path,
                '-i',
                self.test_inventory,
                ansible_playbook,
                '--extra-vars',
                '@%s' % vars_filename
            ],
            state['args'])

    def test_hook_ansible_failed(self):

        self.env.update({
            'TEST_RESPONSE': json.dumps({
                'stdout': 'ansible failed',
                'stderr': 'bad thing happened',
                'returncode': 4
            }),
        })
        returncode, stdout, stderr = self.run_cmd(
            [self.hook_path], self.env, json.dumps(self.data))

        self.assertEqual(0, returncode, stderr)
        self.assertEqual({
            'deploy_stdout': 'ansible failed',
            'deploy_stderr': 'bad thing happened',
            'deploy_status_code': 4,
        }, json.loads(stdout.decode('utf-8')))

        state = self.json_from_file(self.test_state_path)
        ansible_playbook = self.working_dir.join('1234_playbook.yaml')
        vars_filename = self.working_dir.join('1234_variables.json')

        self.assertEqual(
            [
                self.fake_tool_path,
                '-i',
                'localhost,',
                ansible_playbook,
                '--extra-vars',
                '@%s' % vars_filename
            ],
            state['args'])

        # Write 'variables' to file
        variables = self.json_from_file(vars_filename)
        self.assertEqual('bar', variables['foo'])
        self.assertEqual('input', variables['another'])
        self.assertEqual(self.outputs_dir.join('1234'),
                         variables['heat_outputs_path'])

        # Write the executable 'config' to file
        with open(ansible_playbook) as f:
            self.assertEqual('the ansible playbook', f.read())



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy
import json
import os

import fixtures

from tests import common


class HookPuppetTest(common.RunScriptTest):

    data = {
        'id': '1234',
        'creation_time': '2015-07-16T11:40:20',
        'name': 'fake_resource_name',
        'group': 'puppet',
        'options': {
            'enable_hiera': True,
            'enable_facter': True,
            'enable_debug': True,
            'enable_verbose': True,
        },
        'inputs': [
            {'name': 'foo', 'value': 'bar'},
            {'name': 'another', 'value': 'input'}
        ],
        'outputs': [
            {'name': 'first_output'},
            {'name': 'second_output'}
        ],
        'config': 'the puppet script'
    }

    def setUp(self):
        super(HookPuppetTest, self).setUp()
        self.hook_path = self.relative_path(
            __file__,
            '..',
            'heat-config-puppet/install.d/hook-puppet.py')

        self.fake_tool_path = self.relative_path(
            __file__,
            'config-tool-fake.py')

        self.working_dir = self.useFixture(fixtures.TempDir())
        self.outputs_dir = self.useFixture(fixtures.TempDir())
        self.log_dir = self.useFixture(fixtures.TempDir())
        self.hiera_datadir = self.useFixture(fixtures.TempDir())
        self.test_state_path = self.outputs_dir.join('test_state.json')

        self.env = os.environ.copy()
        self.env.update({
            'HEAT_PUPPET_WORKING': self.working_dir.join(),
            'HEAT_PUPPET_OUTPUTS': self.outputs_dir.join(),
            'HEAT_PUPPET_LOGDIR': self.log_dir.join(),
            'HEAT_PUPPET_HIERA_DATADIR': self.hiera_datadir.join(),
            'HEAT_PUPPET_CMD': self.fake_tool_path,
            'TEST_STATE_PATH': self.test_state_path,
        })

    def test_hook(self):

        self.env.update({
            'TEST_RESPONSE': json.dumps({
                'stdout': 'puppet success',
                'stderr': 'thing happened',
                'files': {
                    self.outputs_dir.join('1234.first_output'): 'output 1',
                    self.outputs_dir.join('1234.second_output'): 'output 2',
                }
            }),
        })
        returncode, stdout, stderr = self.run_cmd(
            [self.hook_path], self.env, json.dumps(self.data))

        self.assertEqual(0, returncode, stderr)
        self.assertEqual({
            'deploy_stdout': 'puppet success',
            'deploy_stderr': 'thing happened',
            'deploy_status_code': 0,
            'first_output': 'output 1',
            'second_output': 'output 2',
        }, json.loads(stdout.decode('utf-8')))

        state = self.json_from_file(self.test_state_path)
        puppet_script = self.working_dir.join('1234.pp')
        self.assertEqual(
            [
                self.fake_tool_path,
                'apply',
                '--color',
                'false',
                '--detailed-exitcodes',
                '--logdest',
                'console',
                '--debug',
                '--logdest',
                '/var/log/puppet/heat-debug.log',
                '--verbose',
                '--logdest',
                '/var/log/puppet/heat-verbose.log',
                puppet_script
            ],
            state['args'])

        self.assertEqual('bar', state['env']['FACTER_foo'])
        self.assertEqual('input', state['env']['FACTER_another'])
        self.assertEqual(self.outputs_dir.join('1234'),
                         state['env']['FACTER_heat_outputs_path'])
        with open(puppet_script) as f:
            self.assertEqual('the puppet script', f.read())

    def test_hook_no_debug(self):
        self.data['options']['enable_debug'] = False
        self.env.update({
            'TEST_RESPONSE': json.dumps({
                'stdout': 'success',
                'stderr': '',
            }),
        })
        returncode, stdout, stderr = self.run_cmd(
            [self.hook_path], self.env, json.dumps(self.data))

        state = self.json_from_file(self.test_state_path)
        puppet_script = self.working_dir.join('1234.pp')
        self.assertEqual(
            [
                self.fake_tool_path,
                'apply',
                '--color',
                'false',
                '--detailed-exitcodes',
                '--logdest',
                'console',
                '--verbose',
                '--logdest',
                '/var/log/puppet/heat-verbose.log',
                puppet_script
            ],
            state['args'])
        self.data['options']['enable_debug'] = True

    def test_hook_no_verbose(self):
        self.data['options']['enable_verbose'] = False
        self.env.update({
            'TEST_RESPONSE': json.dumps({
                'stdout': 'success',
                'stderr': '',
            }),
        })
        returncode, stdout, stderr = self.run_cmd(
            [self.hook_path], self.env, json.dumps(self.data))

        state = self.json_from_file(self.test_state_path)
        puppet_script = self.working_dir.join('1234.pp')
        self.assertEqual(
            [
                self.fake_tool_path,
                'apply',
                '--color',
                'false',
                '--detailed-exitcodes',
                '--logdest',
                'console',
                '--debug',
                '--logdest',
                '/var/log/puppet/heat-debug.log',
                puppet_script
            ],
            state['args'])
        self.data['options']['enable_verbose'] = True

    def test_hook_puppet_failed(self):

        self.env.update({
            'TEST_RESPONSE': json.dumps({
                'stdout': 'puppet failed',
                'stderr': 'bad thing happened',
                'returncode': 4
            }),
        })
        returncode, stdout, stderr = self.run_cmd(
            [self.hook_path], self.env, json.dumps(self.data))

        self.assertEqual(0, returncode, stderr)
        self.assertEqual({
            'deploy_stdout': 'puppet failed',
            'deploy_stderr': 'bad thing happened',
            'deploy_status_code': 4,
        }, json.loads(stdout.decode('utf-8')))

        state = self.json_from_file(self.test_state_path)
        puppet_script = self.working_dir.join('1234.pp')
        self.assertEqual(
            [
                self.fake_tool_path,
                'apply',
                '--color',
                'false',
                '--detailed-exitcodes',
                '--logdest',
                'console',
                '--debug',
                '--logdest',
                '/var/log/puppet/heat-debug.log',
                '--verbose',
                '--logdest',
                '/var/log/puppet/heat-verbose.log',
                puppet_script
            ],
            state['args'])

        self.assertEqual('bar', state['env']['FACTER_foo'])
        self.assertEqual('input', state['env']['FACTER_another'])
        self.assertEqual(self.outputs_dir.join('1234'),
                         state['env']['FACTER_heat_outputs_path'])
        with open(puppet_script) as f:
            self.assertEqual('the puppet script', f.read())

    def test_hook_hiera(self):

        self.env.update({
            'TEST_RESPONSE': json.dumps({
                'stdout': 'puppet success',
                'stderr': 'thing happened',
                'files': {
                    self.outputs_dir.join('1234.first_output'): 'output 1',
                    self.outputs_dir.join('1234.second_output'): 'output 2',
                }
            }),
        })
        modulepath = self.working_dir.join()
        data = copy.deepcopy(self.data)
        data['options']['modulepath'] = modulepath
        data['options']['tags'] = 'package,file'
        returncode, stdout, stderr = self.run_cmd(
            [self.hook_path], self.env, json.dumps(data))

        self.assertEqual(0, returncode, stderr)
        self.assertEqual({
            'deploy_stdout': 'puppet success',
            'deploy_stderr': 'thing happened',
            'deploy_status_code': 0,
            'first_output': 'output 1',
            'second_output': 'output 2',
        }, json.loads(stdout.decode('utf-8')))

        state = self.json_from_file(self.test_state_path)
        puppet_script = self.working_dir.join('1234.pp')
        hiera_datafile = self.hiera_datadir.join('heat_config_%s.json'
                                                 % self.data['name'])
        self.assertEqual(
            [
                self.fake_tool_path,
                'apply',
                '--color',
                'false',
                '--detailed-exitcodes',
                '--logdest',
                'console',
                '--modulepath',
                modulepath,
                '--tags',
                'package,file',
                '--debug',
                '--logdest',
                '/var/log/puppet/heat-debug.log',
                '--verbose',
                '--logdest',
                '/var/log/puppet/heat-verbose.log',
                puppet_script
            ],
            state['args'])

        self.assertEqual(self.outputs_dir.join('1234'),
                         state['env']['FACTER_heat_outputs_path'])
        with open(puppet_script) as f:
            self.assertEqual('the puppet script', f.read())
        with open(hiera_datafile) as f:
            self.assertEqual({
                'foo': 'bar',
                'another': 'input',
            }, json.loads(f.read()))



#!/usr/bin/env python
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import logging
import os
import shutil
import six
import subprocess
import sys

DEPLOY_KEYS = ("deploy_server_id",
               "deploy_action",
               "deploy_stack_id",
               "deploy_resource_name",
               "deploy_signal_transport",
               "deploy_signal_id",
               "deploy_signal_verb")
WORKING_DIR = os.environ.get('HEAT_CHEF_WORKING',
                             '/var/lib/heat-config/heat-config-chef')
OUTPUTS_DIR = os.environ.get('HEAT_CHEF_OUTPUTS',
                             '/var/run/heat-config/heat-config-chef')


def prepare_dir(path):
    if not os.path.isdir(path):
        os.makedirs(path, 0o700)


def run_subproc(fn, **kwargs):
    env = os.environ.copy()
    for k, v in kwargs.items():
        env[six.text_type(k)] = v
    try:
        subproc = subprocess.Popen(fn, stdout=subprocess.PIPE,
                                   stderr=subprocess.PIPE,
                                   env=env)
        stdout, stderr = subproc.communicate()
    except OSError as exc:
        ret = -1
        stderr = six.text_type(exc)
        stdout = ""
    else:
        ret = subproc.returncode
    if not ret:
        ret = 0
    return ret, stdout, stderr


def main(argv=sys.argv):
    log = logging.getLogger('heat-config')
    handler = logging.StreamHandler(sys.stderr)
    handler.setFormatter(
        logging.Formatter(
            '[%(asctime)s] (%(name)s) [%(levelname)s] %(message)s'))
    log.addHandler(handler)
    log.setLevel('DEBUG')

    prepare_dir(OUTPUTS_DIR)
    prepare_dir(WORKING_DIR)
    os.chdir(WORKING_DIR)

    c = json.load(sys.stdin)

    client_config = ("log_level :debug\n"
                     "log_location STDOUT\n"
                     "local_mode true\n"
                     "chef_zero.enabled true")

    # configure/set up the kitchen
    kitchen = c['options'].get('kitchen')
    kitchen_path = c['options'].get('kitchen_path', os.path.join(WORKING_DIR,
                                                                 "kitchen"))
    cookbook_path = os.path.join(kitchen_path, "cookbooks")
    role_path = os.path.join(kitchen_path, "roles")
    environment_path = os.path.join(kitchen_path, "environments")
    client_config += "\ncookbook_path '%s'" % cookbook_path
    client_config += "\nrole_path '%s'" % role_path
    client_config += "\nenvironment_path '%s'" % environment_path
    if kitchen:
        log.debug("Cloning kitchen from %s", kitchen)
        # remove the existing kitchen on update so we get a fresh clone
        dep_action = next((input['value'] for input in c['inputs']
                           if input['name'] == "deploy_action"), None)
        if dep_action == "UPDATE":
            shutil.rmtree(kitchen_path, ignore_errors=True)
        cmd = ["git", "clone", kitchen, kitchen_path]
        ret, out, err = run_subproc(cmd)
        if ret != 0:
            log.error("Error cloning kitchen from %s into %s: %s", kitchen,
                      kitchen_path, err)
            json.dump({'deploy_status_code': ret,
                       'deploy_stdout': out,
                       'deploy_stderr': err},
                      sys.stdout)
            return 0

    # write the json attributes
    ret, out, err = run_subproc(['hostname', '-f'])
    if ret == 0:
        fqdn = out.strip()
    else:
        err = "Could not determine hostname with hostname -f"
        json.dump({'deploy_status_code': ret,
                   'deploy_stdout': "",
                   'deploy_stderr': err}, sys.stdout)
        return 0
    node_config = {}
    for input in c['inputs']:
        if input['name'] == 'environment':
            client_config += "\nenvironment '%s'" % input['value']
        elif input['name'] not in DEPLOY_KEYS:
            node_config.update({input['name']: input['value']})
    node_config.update({"run_list": json.loads(c['config'])})
    node_path = os.path.join(WORKING_DIR, "node")
    prepare_dir(node_path)
    node_file = os.path.join(node_path, "%s.json" % fqdn)
    with os.fdopen(os.open(node_file, os.O_CREAT | os.O_WRONLY, 0o600),
                   'w') as f:
        f.write(json.dumps(node_config, indent=4, separators=(',', ': ')))
    client_config += "\nnode_path '%s'" % node_path

    # write out the completed client config
    config_path = os.path.join(WORKING_DIR, "client.rb")
    with os.fdopen(os.open(config_path, os.O_CREAT | os.O_WRONLY, 0o600),
                   'w') as f:
        f.write(client_config)

    # run chef
    heat_outputs_path = os.path.join(OUTPUTS_DIR, c['id'])
    cmd = ['chef-client', '-z', '--config', config_path, "-j", node_file]
    ret, out, err = run_subproc(cmd, heat_outputs_path=heat_outputs_path)
    resp = {'deploy_status_code': ret,
            'deploy_stdout': out,
            'deploy_stderr': err}
    log.debug("Chef output: %s", out)
    if err:
        log.error("Chef return code %s:\n%s", ret, err)
    for output in c.get('outputs', []):
        output_name = output['name']
        try:
            with open('%s.%s' % (heat_outputs_path, output_name)) as out:
                resp[output_name] = out.read()
        except IOError:
            pass
    json.dump(resp, sys.stdout)


if __name__ == '__main__':
    sys.exit(main(sys.argv))



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy
import imp
import io
import json
import logging
import mock
import six
import sys

from tests import common

log = logging.getLogger('test_hook_chef')


@mock.patch("os.chdir")
@mock.patch("os.makedirs")
@mock.patch('subprocess.Popen')
class HookChefTest(common.RunScriptTest):

    data = {
        'id': 'fake_stack',
        'name': 'fake_resource_name',
        'group': 'chef',
        'inputs': [
            {'name': 'fooval', 'value': {'bar': 'baz'}},
            {'name': 'barval', 'value': {'foo': 'biff'}},
            {'name': "deploy_server_id", 'value': 'foo'},
            {'name': "deploy_action", 'value': 'foo'},
            {'name': "deploy_stack_id", 'value': 'foo'},
            {'name': "deploy_resource_name", 'value': 'foo'},
            {'name': "deploy_signal_transport", 'value': 'foo'},
            {'name': "deploy_signal_id", 'value': 'foo'},
            {'name': "deploy_signal_verb", 'value': 'foo'}
        ],
        'options': {},
        'outputs': [
            {'name': 'first_output'},
            {'name': 'second_output'}
        ],
        'config': None
    }

    def setUp(self):
        super(HookChefTest, self).setUp()
        self.hook_path = self.relative_path(
            __file__,
            '..',
            'heat-config-chef/install.d/hook-chef.py')
        if six.PY2:
            sys.stdin = io.BytesIO()
            sys.stdout = io.BytesIO()
        else:
            sys.stdin = io.StringIO()
            sys.stdout = io.StringIO()

    def tearDown(self):
        super(HookChefTest, self).tearDown()
        sys.stdin = sys.__stdin__
        sys.stdout = sys.__stdout__

    def get_module(self):
        try:
            imp.acquire_lock()
            return imp.load_source("hook_chef", self.hook_path)
        finally:
            imp.release_lock()

    def test_hook(self, mock_popen, mock_mkdirs, mock_chdir):
        data = copy.deepcopy(self.data)
        data['config'] = '["recipe[apache]"]'
        hook_chef = self.get_module()
        json.dump(data, sys.stdin)
        sys.stdin.seek(0)
        mock_subproc = mock.Mock()
        mock_popen.return_value = mock_subproc
        mock_subproc.communicate.return_value = ("out", "err")
        mock_subproc.returncode = 0
        with mock.patch("os.fdopen", mock.mock_open()) as mfdopen:
            with mock.patch("os.open", mock.mock_open()):
                hook_chef.main(json.dumps(data))
                exp_node = {
                    'barval': {'foo': 'biff'},
                    'fooval': {u'bar': u'baz'},
                    'run_list': [u'recipe[apache]']
                }
                exp_cfg = ("log_level :debug\n"
                           "log_location STDOUT\n"
                           "local_mode true\n"
                           "chef_zero.enabled true\n"
                           "cookbook_path '/var/lib/heat-config/"
                           "heat-config-chef/kitchen/cookbooks'\n"
                           "role_path '/var/lib/heat-config/"
                           "heat-config-chef/kitchen/roles'\n"
                           "environment_path '/var/lib/heat-config/"
                           "heat-config-chef/kitchen/environments'\n"
                           "node_path '/var/lib/heat-config/"
                           "heat-config-chef/node'")
                mfdopen.return_value.write.assert_any_call(exp_cfg)
                args = mfdopen.return_value.write.mock_calls[0][1][0]
                self.assertEqual(exp_node, json.loads(args))
        calls = [
            mock.call(['hostname', '-f'], env=mock.ANY, stderr=mock.ANY,
                      stdout=mock.ANY),
            mock.call([
                'chef-client', '-z', '--config',
                '/var/lib/heat-config/heat-config-chef/client.rb', '-j',
                '/var/lib/heat-config/heat-config-chef/node/out.json'],
                env=mock.ANY, stderr=mock.ANY, stdout=mock.ANY)
        ]
        mock_popen.assert_has_calls(calls, any_order=True)
        self.assertEqual({"deploy_status_code": 0,
                          "deploy_stdout": "out",
                          "deploy_stderr": "err"},
                         json.loads(sys.stdout.getvalue()))

    def test_hook_with_kitchen(self, mock_popen, mock_mkdirs, mock_chdir):
        data = copy.deepcopy(self.data)
        data['config'] = '["recipe[apache]"]'
        data['options'] = {
            "kitchen": "https://github.com/fake.git",
            "kitchen_path": "/opt/heat/chef/kitchen"
        }
        sys.stdin.write(json.dumps(data))
        hook_chef = self.get_module()
        sys.stdin.seek(0)
        mock_subproc = mock.Mock()
        mock_popen.return_value = mock_subproc
        mock_subproc.communicate.return_value = ("out", "err")
        mock_subproc.returncode = 0
        with mock.patch("os.fdopen", mock.mock_open()) as mfdopen:
            with mock.patch("os.open", mock.mock_open()):
                hook_chef.main(json.dumps(data))
                exp_cfg = ("log_level :debug\n"
                           "log_location STDOUT\n"
                           "local_mode true\n"
                           "chef_zero.enabled true\n"
                           "cookbook_path '/opt/heat/chef/kitchen/"
                           "cookbooks'\n"
                           "role_path '/opt/heat/chef/kitchen/roles'\n"
                           "environment_path '/opt/heat/chef/kitchen/"
                           "environments'\n"
                           "node_path '/var/lib/heat-config/heat-config-chef"
                           "/node'")
                mfdopen.return_value.write.assert_any_call(exp_cfg)
        calls = [
            mock.call(['git', 'clone', "https://github.com/fake.git",
                       "/opt/heat/chef/kitchen"], env=mock.ANY,
                      stderr=mock.ANY, stdout=mock.ANY),
            mock.call(['hostname', '-f'], env=mock.ANY, stderr=mock.ANY,
                      stdout=mock.ANY),
            mock.call([
                'chef-client', '-z', '--config',
                '/var/lib/heat-config/heat-config-chef/client.rb', '-j',
                '/var/lib/heat-config/heat-config-chef/node/out.json'],
                env=mock.ANY, stderr=mock.ANY, stdout=mock.ANY)
        ]
        mock_popen.assert_has_calls(calls, any_order=True)
        self.assertEqual({"deploy_status_code": 0,
                          "deploy_stdout": "out",
                          "deploy_stderr": "err"},
                         json.loads(sys.stdout.getvalue()))

    def test_hook_environment(self, mock_popen, mock_mkdirs, mock_chdir):
        data = copy.deepcopy(self.data)
        data['config'] = '["recipe[apache]"]'
        data['inputs'].append({'name': 'environment',
                               'value': 'production'})
        hook_chef = self.get_module()
        sys.stdin.write(json.dumps(data))
        sys.stdin.seek(0)
        mock_subproc = mock.Mock()
        mock_popen.return_value = mock_subproc
        mock_subproc.communicate.return_value = ("out", "err")
        mock_subproc.returncode = 0
        with mock.patch("os.fdopen", mock.mock_open()) as mfdopen:
            with mock.patch("os.open", mock.mock_open()):
                hook_chef.main(json.dumps(data))
                exp_node = {
                    'barval': {'foo': 'biff'},
                    'fooval': {u'bar': u'baz'},
                    'run_list': [u'recipe[apache]']
                }
                exp_cfg = ("log_level :debug\n"
                           "log_location STDOUT\n"
                           "local_mode true\n"
                           "chef_zero.enabled true\n"
                           "cookbook_path '/var/lib/heat-config/"
                           "heat-config-chef/kitchen/cookbooks'\n"
                           "role_path '/var/lib/heat-config/"
                           "heat-config-chef/kitchen/roles'\n"
                           "environment_path '/var/lib/heat-config/"
                           "heat-config-chef/kitchen/environments'\n"
                           "environment 'production'\n"
                           "node_path '/var/lib/heat-config/"
                           "heat-config-chef/node'")
                mfdopen.return_value.write.assert_any_call(exp_cfg)
                args = mfdopen.return_value.write.mock_calls[0][1][0]
                self.assertEqual(exp_node, json.loads(args))



#!/usr/bin/env python
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import logging
import os
import subprocess
import sys

WORKING_DIR = os.environ.get('HEAT_SCRIPT_WORKING',
                             '/var/lib/heat-config/heat-config-script')
OUTPUTS_DIR = os.environ.get('HEAT_SCRIPT_OUTPUTS',
                             '/var/run/heat-config/heat-config-script')


def prepare_dir(path):
    if not os.path.isdir(path):
        os.makedirs(path, 0o700)


def main(argv=sys.argv):
    log = logging.getLogger('heat-config')
    handler = logging.StreamHandler(sys.stderr)
    handler.setFormatter(
        logging.Formatter(
            '[%(asctime)s] (%(name)s) [%(levelname)s] %(message)s'))
    log.addHandler(handler)
    log.setLevel('DEBUG')

    prepare_dir(OUTPUTS_DIR)
    prepare_dir(WORKING_DIR)
    os.chdir(WORKING_DIR)

    c = json.load(sys.stdin)

    env = os.environ.copy()
    for input in c['inputs']:
        input_name = input['name']
        value = input.get('value', '')
        if isinstance(value, dict) or isinstance(value, list):
            env[input_name] = json.dumps(value)
        else:
            env[input_name] = value
        log.info('%s=%s' % (input_name, env[input_name]))

    fn = os.path.join(WORKING_DIR, c['id'])
    heat_outputs_path = os.path.join(OUTPUTS_DIR, c['id'])
    env['heat_outputs_path'] = heat_outputs_path

    with os.fdopen(os.open(fn, os.O_CREAT | os.O_WRONLY, 0o700), 'w') as f:
        f.write(c.get('config', ''))

    log.debug('Running %s' % fn)
    subproc = subprocess.Popen([fn], stdout=subprocess.PIPE,
                               stderr=subprocess.PIPE, env=env)
    stdout, stderr = subproc.communicate()

    log.info(stdout)
    log.debug(stderr)

    if subproc.returncode:
        log.error("Error running %s. [%s]\n" % (fn, subproc.returncode))
    else:
        log.info('Completed %s' % fn)

    response = {}

    for output in c.get('outputs') or []:
        output_name = output['name']
        try:
            with open('%s.%s' % (heat_outputs_path, output_name)) as out:
                response[output_name] = out.read()
        except IOError:
            pass

    response.update({
        'deploy_stdout': stdout.decode('utf-8', 'replace'),
        'deploy_stderr': stderr.decode('utf-8', 'replace'),
        'deploy_status_code': subproc.returncode,
    })

    json.dump(response, sys.stdout)

if __name__ == '__main__':
    sys.exit(main(sys.argv))



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import logging
import os
import tempfile
import yaml

import fixtures

from tests import common

log = logging.getLogger('test_hook_json_file')


class HookJsonFileConfigTest(common.RunScriptTest):

    def setUp(self):
        super(HookJsonFileConfigTest, self).setUp()
        self.hook_path = self.relative_path(
            __file__,
            '..',
            'heat-config-json-file/install.d/hook-json-file.py')

        conf_dir = self.useFixture(fixtures.TempDir()).join()
        self.conf = tempfile.NamedTemporaryFile(
            mode='w', dir=conf_dir, delete=False).name
        os.unlink(self.conf)

        self.env = os.environ.copy()
        self.data = {
            'id': 'test_json_file',
            'name': 'fake_resource_name',
            'group': 'json-file',
            'config': {
                self.conf: {
                  'command': 'foo'
                }
            }
        }

    def test_hook(self):

        returncode, stdout, stderr = self.run_cmd(
            [self.hook_path], self.env, json.dumps(self.data))

        self.assertEqual(0, returncode, stderr)
        ret = yaml.safe_load(stdout)
        self.assertIsNotNone(ret['deploy_stderr'])
        self.assertEqual('', ret['deploy_stdout'])
        self.assertEqual(0, ret['deploy_status_code'])

        with open(os.path.join(self.conf)) as data:
            self.assertEqual("{\n    \"command\": \"foo\"\n}", data.read())



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import fixtures
import json
import logging
import os
import tempfile
import yaml

from tests import common

log = logging.getLogger('test_hook_hiera_config')

HIERA_CONFIG_BASE = """
---
:backends:
  - json
:json:
  :datadir: %(datadir)s
:hierarchy:
  - %(datafile)s
"""


class HookHieraTest(common.RunScriptTest):

    data = {
        'id': 'test_hiera',
        'name': 'fake_resource_name',
        'group': 'hiera',
        'config': {
            'hierarchy': ['compute'],
            'datafiles': {
                'compute': {'foo': 'bar'}
            }
        }
    }

    def setUp(self):
        super(HookHieraTest, self).setUp()
        self.hook_path = self.relative_path(
            __file__,
            '..',
            'heat-config-hiera/install.d/hook-hiera.py')

        self.hieradata_dir = self.useFixture(fixtures.TempDir()).join()
        conf_dir = self.useFixture(fixtures.TempDir()).join()
        self.conf = tempfile.NamedTemporaryFile(
            dir=conf_dir, mode='w', delete=False).name
        os.unlink(self.conf)

        self.env = os.environ.copy()
        self.env.update({
            'HEAT_HIERA_CONFIG': self.conf,
            'HEAT_PUPPET_HIERA_DATADIR': self.hieradata_dir,
        })

    def test_hook(self):

        returncode, stdout, stderr = self.run_cmd(
            [self.hook_path], self.env, json.dumps(self.data))

        self.assertEqual(0, returncode, stderr)
        ret = yaml.safe_load(stdout)
        self.assertIsNotNone(ret['deploy_stderr'])
        self.assertEqual('', ret['deploy_stdout'])
        self.assertEqual(0, ret['deploy_status_code'])

        conf_data = HIERA_CONFIG_BASE % {'datadir': self.hieradata_dir,
                                         'datafile': 'compute'}
        with open(self.conf) as conf_file:
            self.assertEqual(conf_data, conf_file.read())

        with open(os.path.join(self.hieradata_dir, 'compute.json')) as data:
            self.assertEqual("{\n    \"foo\": \"bar\"\n}", data.read())



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import os
import tempfile

import fixtures
from testtools import matchers

from tests import common


class HeatConfigKubeletORCTest(common.RunScriptTest):

    fake_hooks = ['kubelet']

    data = [{
        "id": "abcdef001",
        "group": "kubelet",
        "name": "mysql",
        "config": {
            "version": "v1beta2",
            "volumes": [{
                "name": "mariadb-data"
            }],
            "containers": [{
                "image": "mariadb_image",
                "volumeMounts": [{
                    "mountPath": "/var/lib/mysql",
                    "name": "mariadb-data"
                }],
                "name": "mariadb",
                "env": [{
                    "name": "DB_ROOT_PASSWORD",
                    "value": "mariadb_password"
                }],
                "ports": [{
                    "containerPort": 3306
                }]
            }]}
    }, {
        "id": "abcdef002",
        "group": "kubelet",
        "name": "rabbitmq",
        "config": {
            "version": "v1beta2",
            "containers": [{
                "image": "rabbitmq_image",
                "name": "rabbitmq",
                "ports": [{
                    "containerPort": 5672
                }]
            }]
        }
    }, {
        "id": "abcdef003",
        "group": "kubelet",
        "name": "heat_api_engine",
        "config": {
            "version": "v1beta2",
            "containers": [{
                "image": "heat_engine_image",
                "name": "heat-engine",
                "env": [{
                    "name": "DB_ROOT_PASSWORD",
                    "value": "mariadb_password"
                }, {
                    "name": "HEAT_DB_PASSWORD",
                    "value": "heatdb_password"
                }, {
                    "name": "HEAT_KEYSTONE_PASSWORD",
                    "value": "password"
                }]
            }, {
                "image": "heat_api_image",
                "name": "heat-api",
                "ports": [{
                    "containerPort": 8004
                }]
            }]
        }
    }]

    def setUp(self):
        super(HeatConfigKubeletORCTest, self).setUp()

        self.fake_hook_path = self.relative_path(__file__, 'hook-fake.py')

        self.heat_config_kubelet_path = self.relative_path(
            __file__,
            '..',
            'heat-config-kubelet/os-refresh-config/configure.d/'
            '50-heat-config-kubelet')

        self.manifests_dir = self.useFixture(fixtures.TempDir())

        with open(self.fake_hook_path) as f:
            fake_hook = f.read()

        for hook in self.fake_hooks:
            hook_name = self.manifests_dir.join(hook)
            with open(hook_name, 'w') as f:
                os.utime(hook_name, None)
                f.write(fake_hook)
                f.flush()
            os.chmod(hook_name, 0o755)

    def write_config_file(self, data):
        config_file = tempfile.NamedTemporaryFile(mode='w')
        config_file.write(json.dumps(data))
        config_file.flush()
        return config_file

    def test_run_heat_config(self):

        with self.write_config_file(self.data) as config_file:

            env = os.environ.copy()
            env.update({
                'HEAT_KUBELET_MANIFESTS': self.manifests_dir.join(),
                'HEAT_SHELL_CONFIG': config_file.name
            })
            returncode, stdout, stderr = self.run_cmd(
                [self.heat_config_kubelet_path], env)

            self.assertEqual(0, returncode, stderr)

        for config in self.data:
            manifest_name = '%s.json' % config['id']
            manifest_path = self.manifests_dir.join(manifest_name)
            self.assertThat(manifest_path, matchers.FileExists())

            # manifest file should match manifest config
            self.assertEqual(config['config'],
                             self.json_from_file(manifest_path))



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import os

import fixtures

from tests import common


class HookCfnInitTest(common.RunScriptTest):

    data = {
        'group': 'cfn-init',
        'inputs': [],
        'config': {'foo': 'bar'}
    }

    def setUp(self):
        super(HookCfnInitTest, self).setUp()
        self.hook_path = self.relative_path(
            __file__,
            '..',
            'heat-config-cfn-init/install.d/hook-cfn-init.py')

        self.fake_tool_path = self.relative_path(
            __file__,
            'config-tool-fake.py')

        self.metadata_dir = self.useFixture(fixtures.TempDir())
        # use the temp dir to store the fake config tool state too
        self.test_state_path = self.metadata_dir.join('test_state.json')
        self.env = os.environ.copy()
        self.env.update({
            'HEAT_CFN_INIT_LAST_METADATA_DIR': self.metadata_dir.join(),
            'HEAT_CFN_INIT_CMD': self.fake_tool_path,
            'TEST_STATE_PATH': self.test_state_path,
        })

    def test_hook(self):

        self.env.update({
            'TEST_RESPONSE': json.dumps({
                'stdout': 'cfn-init success',
                'stderr': 'thing happened'
            }),
        })
        returncode, stdout, stderr = self.run_cmd(
            [self.hook_path], self.env, json.dumps(self.data))

        self.assertEqual(0, returncode, stderr)
        self.assertEqual({
            'deploy_stdout': 'cfn-init success',
            'deploy_stderr': 'thing happened',
            'deploy_status_code': 0
        }, json.loads(stdout.decode('utf-8')))

        # assert last_metadata was written with cfn-init metadata
        self.assertEqual(
            {'AWS::CloudFormation::Init': {'foo': 'bar'}},
            self.json_from_file(self.metadata_dir.join('last_metadata')))

        # assert cfn-init was called with no args
        self.assertEqual(
            [self.fake_tool_path],
            self.json_from_file(self.test_state_path)['args'])

    def test_hook_cfn_init_failed(self):

        self.env.update({
            'TEST_RESPONSE': json.dumps({
                'stderr': 'bad thing happened',
                'returncode': 1
            }),
        })
        returncode, stdout, stderr = self.run_cmd(
            [self.hook_path], self.env, json.dumps(self.data))

        self.assertEqual(0, returncode, stderr)
        self.assertEqual({
            'deploy_stdout': '',
            'deploy_stderr': 'bad thing happened',
            'deploy_status_code': 1
        }, json.loads(stdout.decode('utf-8')))

        self.assertEqual(
            {'AWS::CloudFormation::Init': {'foo': 'bar'}},
            self.json_from_file(self.metadata_dir.join('last_metadata')))

        # assert cfn-init was called with no args
        self.assertEqual(
            [self.fake_tool_path],
            self.json_from_file(self.test_state_path)['args'])

    def test_hook_invalid_json(self):

        returncode, stdout, stderr = self.run_cmd(
            [self.hook_path], self.env, "{::::")

        self.assertEqual(1, returncode, stderr)



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import os

import fixtures

from tests import common


class HookDockerComposeTest(common.RunScriptTest):
    data = {
        "name": "abcdef001",
        "group": "docker-compose",
        "inputs": [
            {
                "name": "env_files",
                "value": u'[ { "file_name": "./common.env", '
                         u'"content": "xxxxx" }, '
                         u'{ "file_name": "./test.env", '
                         u'"content": "yyyy" }, '
                         u'{ "file_name": "./test1.env", '
                         u'"content": "zzz" } ]'
            }
        ],
        "config": {
            "web": {
                "name": "x",
                "env_file": [
                    "./common.env",
                    "./test.env"
                ]
            },
            "db": {
                "name": "y",
                "env_file": "./test1.env"
            }
        }
    }

    data_without_input = {
        "name": "abcdef001",
        "group": "docker-compose",
        "inputs": [],
        "config": {
            "web": {
                "name": "x",
                "env_file": [
                    "./common.env",
                    "./test.env"
                ]
            },
            "db": {
                "name": "y",
                "env_file": "./test1.env"
            }
        }
    }

    def setUp(self):
        super(HookDockerComposeTest, self).setUp()
        self.hook_path = self.relative_path(
            __file__,
            '..',
            'heat-config-docker-compose/install.d/hook-docker-compose.py')

        self.fake_tool_path = self.relative_path(
            __file__,
            'config-tool-fake.py')

        self.working_dir = self.useFixture(fixtures.TempDir())
        self.outputs_dir = self.useFixture(fixtures.TempDir())
        self.test_state_path = self.outputs_dir.join('test_state.json')

        self.env = os.environ.copy()
        self.env.update({
            'HEAT_DOCKER_COMPOSE_WORKING': self.working_dir.join(),
            'HEAT_DOCKER_COMPOSE_CMD': self.fake_tool_path,
            'TEST_STATE_PATH': self.test_state_path,
        })

    def test_hook(self):

        self.env.update({
            'TEST_RESPONSE': json.dumps({
                'stdout': '',
                'stderr': 'Creating abcdef001_db_1...'
            })
        })
        returncode, stdout, stderr = self.run_cmd(
            [self.hook_path], self.env, json.dumps(self.data))

        self.assertEqual(0, returncode, stderr)

        self.assertEqual({
            'deploy_stdout': '',
            'deploy_stderr': 'Creating abcdef001_db_1...',
            'deploy_status_code': 0
        }, json.loads(stdout.decode('utf-8')))

        state = self.json_from_file(self.test_state_path)
        self.assertEqual(
            [
                self.fake_tool_path,
                'up',
                '-d',
                '--no-build',
            ],
            state['args'])

    def test_hook_without_inputs(self):

        self.env.update({
            'TEST_RESPONSE': json.dumps({
                'stdout': '',
                'stderr': 'env_file_not found...',
                'returncode': 1
            })
        })
        returncode, stdout, stderr = self.run_cmd(
            [self.hook_path], self.env, json.dumps(self.data_without_input))

        self.assertEqual({
            'deploy_stdout': '',
            'deploy_stderr': 'env_file_not found...',
            'deploy_status_code': 1
        }, json.loads(stdout.decode('utf-8')))

        state = self.json_from_file(self.test_state_path)
        self.assertEqual(
            [
                self.fake_tool_path,
                'up',
                '-d',
                '--no-build',
            ],
            state['args'])

    def test_hook_failed(self):

        self.env.update({
            'TEST_RESPONSE': json.dumps({
                'stdout': '',
                'stderr': 'Error: image library/xxx:latest not found',
                'returncode': 1
            })
        })
        returncode, stdout, stderr = self.run_cmd(
            [self.hook_path], self.env, json.dumps(self.data))

        self.assertEqual({
            'deploy_stdout': '',
            'deploy_stderr': 'Error: image library/xxx:latest not found',
            'deploy_status_code': 1
        }, json.loads(stdout.decode('utf-8')))

        state = self.json_from_file(self.test_state_path)
        self.assertEqual(
            [
                self.fake_tool_path,
                'up',
                '-d',
                '--no-build',
            ],
            state['args'])



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import fixtures
import json
import logging
import os
import yaml

from tests import common

log = logging.getLogger('test_hook_salt')

slsok = """
testit:
  environ.setenv:
  - name: does_not_matter
  - value:
      foo: {{ opts['fooval'] }}
      bar: {{ opts['barval'] }}
"""

slsfail = """
failure:
  test.echo:
  - text: I don't work
"""

slsnotallowed = """
default to accept:
  iptables.set_policy:
    - chain: INPUT
    - policy: REJECT
"""


class HookSaltTest(common.RunScriptTest):

    data = {
        'id': 'fake_stack',
        'name': 'fake_resource_name',
        'group': 'salt',
        'inputs': [
            {'name': 'fooval', 'value': 'bar'},
            {'name': 'barval', 'value': 'foo'}
        ],
        'outputs': [
            {'name': 'first_output'},
            {'name': 'second_output'}
        ],
        'config': None
    }

    def setUp(self):
        super(HookSaltTest, self).setUp()
        self.hook_path = self.relative_path(
            __file__,
            '..',
            'heat-config-salt/install.d/hook-salt.py')

        self.working_dir = self.useFixture(fixtures.TempDir())
        self.minion_config_dir = self.useFixture(fixtures.TempDir())
        self.minion_cach_dir = self.useFixture(fixtures.TempDir())

        self.minion_conf = self.minion_config_dir.join("minion")

        self.env = os.environ.copy()
        self.env.update({
            'HEAT_SALT_WORKING': self.working_dir.join(),
            'SALT_MINION_CONFIG': self.minion_conf
        })

        with open(self.minion_conf, "w+") as conf_file:
            conf_file.write("cachedir: %s\n" % self.minion_cach_dir.join())
            conf_file.write("log_level: DEBUG\n")

    def test_hook(self):

        self.data['config'] = slsok

        returncode, stdout, stderr = self.run_cmd(
            [self.hook_path], self.env, json.dumps(self.data))

        self.assertEqual(0, returncode, stderr)
        ret = yaml.safe_load(stdout)
        self.assertEqual(0, ret['deploy_status_code'])
        self.assertIsNone(ret['deploy_stderr'])
        self.assertIsNotNone(ret['deploy_stdout'])
        resp = yaml.safe_load(ret['deploy_stdout'])
        self.assertTrue(list(resp.values())[0]['result'])
        self.assertEqual({'bar': 'foo', 'foo': 'bar'},
                         list(resp.values())[0]['changes'])

    def test_hook_salt_failed(self):

        self.data['config'] = slsfail

        returncode, stdout, stderr = self.run_cmd(
            [self.hook_path], self.env, json.dumps(self.data))

        self.assertEqual(0, returncode)
        self.assertIsNotNone(stderr)
        self.assertIsNotNone(stdout)
        jsonout = json.loads(stdout.decode('utf-8'))
        self.assertIsNone(jsonout.get("deploy_stdout"),
                          jsonout.get("deploy_stdout"))
        self.assertEqual(2, jsonout.get("deploy_status_code"))
        self.assertIsNotNone(jsonout.get("deploy_stderr"))
        self.assertIn("was not found in SLS", jsonout.get("deploy_stderr"))

    def test_hook_salt_retcode(self):

        self.data['config'] = slsnotallowed

        returncode, stdout, stderr = self.run_cmd(
            [self.hook_path], self.env, json.dumps(self.data))

        self.assertEqual(0, returncode, stderr)
        self.assertIsNotNone(stdout)
        self.assertIsNotNone(stderr)
        ret = json.loads(stdout.decode('utf-8'))
        self.assertIsNone(ret['deploy_stdout'])
        self.assertIsNotNone(ret['deploy_stderr'])
        resp = list(yaml.safe_load(ret['deploy_stderr']).values())[0]
        self.assertFalse(resp['result'])
